@@ -1,21 +1,24 @@
 /*
  * Standard routines for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -83,7 +86,7 @@ const char	*str;	/* constant string to display */
 #endif
 	printf(_("Please report this bug to the maintainers,\n"));
 	printf(_("along with the entry sequence that caused it.\n"));
-#if	!SILENT
+#if	!LIBRARY
 	printf(_("Type \"help bugs\" for info on how to report bugs found in this program.\n"));
 #endif
 	longjmp(jmp_save, 13);	/* Abort the current operation with the critical error number 13. */
@@ -109,20 +112,22 @@ check_err()
 		break;
 	case ERANGE:
 		errno = 0;
-		error(_("Overflow error in constant."));
+		error(_("Floating point constant out of range."));
 		longjmp(jmp_save, 2);
 		break;
 	}
 }
 
 /*
- * Get the screen (window) width and height from the operating system.
+ * Get the current screen (window) width and height from the operating system.
  *
- * Sets the global integers screen_columns and screen_rows.
+ * Return true if the global integers screen_columns and/or screen_rows were set.
  */
-void
+int
 get_screen_size()
 {
+	int	rv = false;
+
 #if	UNIX || CYGWIN
 	struct winsize	ws;
 
@@ -131,15 +136,19 @@ get_screen_size()
 	if (ioctl(1, TIOCGWINSZ, &ws) >= 0) {
 		if (ws.ws_col > 0) {
 			screen_columns = ws.ws_col;
+			rv = true;
 		}
 		if (ws.ws_row > 0) {
 			screen_rows = ws.ws_row;
+			rv = true;
 		}
 	}
 #else
 	screen_columns = STANDARD_SCREEN_COLUMNS;
 	screen_rows = STANDARD_SCREEN_ROWS;
+	rv = true;
 #endif
+	return rv;
 }
 
 /*
@@ -179,12 +188,12 @@ init_mem()
  *
  * This routine is usually not needed, because when a program exits,
  * all the memory it allocated is released by the operating system.
- * Inclusion of this routine was requested by Tam Hanna for Symbian OS.
+ * Inclusion of this routine was requested by Tam Hanna for use with Symbian OS.
  */
 void
 free_mem()
 {
-	int i;
+	int	i;
 
 	clear_all();
 
@@ -249,6 +258,7 @@ clean_up()
 #endif
 		gfp = default_out;
 	}
+	gfp_filename = NULL;
 	for (i = 0; i < n_equations; i++) {
 		if (n_lhs[i] <= 0) {
 			n_lhs[i] = 0;
@@ -514,6 +524,28 @@ long	v;	/* standard Mathomatic variable */
 	return false;
 }
 
+/*
+ * Replace all occurrences of variable from_v with to_v in an equation space.
+ */
+void
+rename_var_in_es(en, from_v, to_v)
+int	en;	 	/* equation space number */
+long	from_v, to_v;	/* Mathomatic variables */
+{
+	int	i;
+
+	if (n_lhs[en]) {
+		for (i = 0; i < n_lhs[en]; i += 2)
+			if (lhs[en][i].kind == VARIABLE
+			    && lhs[en][i].token.variable == from_v)
+				lhs[en][i].token.variable = to_v;
+		for (i = 0; i < n_rhs[en]; i += 2)
+			if (rhs[en][i].kind == VARIABLE
+			    && rhs[en][i].token.variable == from_v)
+				rhs[en][i].token.variable = to_v;
+	}
+}
+
 /*
  * Substitute every instance of "v" in "equation" with "expression".
  *
@@ -556,7 +588,7 @@ long		v;		/* variable to substitute with expression */
 }
 
 /*
- * Return the minimum level encountered in a Mathomatic "expression".
+ * Return the base (minimum) parentheses level encountered in a Mathomatic "expression".
  */
 int
 min_level(expression, n)
@@ -566,12 +598,15 @@ int		n;		/* expression length */
 	int		min1;
 	token_type	*p1, *ep;
 
-	if (n <= 1) {
-		if (n <= 0) {
-			return 1;
-		} else {
-			return expression[0].level;
-		}
+	switch (n) {
+	case 1:
+		return expression[0].level;
+	case 3:
+		return expression[1].level;
+	default:
+		if (n <= 0 || (n & 1) != 1)
+			error_bug("Invalid expression length in call to min_level().");
+		break;
 	}
 	min1 = expression[1].level;
 	ep = &expression[n];
@@ -720,7 +755,7 @@ char	*string;	/* storage for input string */
 int	n;		/* maximum size of "string" in bytes */
 {
 #if	LIBRARY
-	error(_("Missing command line argument."));
+	error(_("Library usage error. Input requested, possibly due to missing command-line argument."));
 	return NULL;
 #else
 	int	i;
@@ -749,6 +784,7 @@ int	n;		/* maximum size of "string" in bytes */
 		}
 	} else {
 		printf("%s", prompt_str);
+		fflush(stdout);
 		if (fgets(string, n, stdin) == NULL) {
 			if (!quiet_mode)
 				printf(_("\nEnd of input.\n"));
@@ -757,6 +793,7 @@ int	n;		/* maximum size of "string" in bytes */
 	}
 #else
 	printf("%s", prompt_str);
+	fflush(stdout);
 	if (fgets(string, n, stdin) == NULL) {
 		if (!quiet_mode)
 			printf(_("\nEnd of input.\n"));
@@ -783,6 +820,7 @@ int	n;		/* maximum size of "string" in bytes */
  * Display the prompt in prompt_str[] and wait for "y" or "n" followed by Enter.
  *
  * Return true if "y".
+ * Return false if "n" or not interactive.
  */
 int
 get_yes_no()
@@ -790,6 +828,9 @@ get_yes_no()
 	char	*cp;
 	char	buf[MAX_CMD_LEN];
 
+	if (!isatty(0)) {
+		return false;
+	}
 	for (;;) {
 		if ((cp = get_string(buf, sizeof(buf))) == NULL) {
 			return false;
@@ -824,7 +865,17 @@ int	en;	/* equation space number the result is in */
 		factor_int_sub(en);
 	}
 	free_result_str();
+#if	0	/* Set this to 1 to have multi-line, fraction format output from most Mathomatic commands in library mode. */
+	if (display2d) {
+		result_str = flist_equation_string(en);
+		if (result_str == NULL)
+			result_str = list_equation(en, false);
+	} else {
+		result_str = list_equation(en, false);
+	}
+#else	/* Allow feeding command output to the next command's input. */
 	result_str = list_equation(en, false);
+#endif
 	result_en = en;
 	if (gfp == stdout) {
 		return(result_str != NULL);
@@ -860,7 +911,8 @@ char	*cp;
 /*
  * Process an equation number range given in text string "*cpp".
  * Skip past all spaces and update "*cpp" to point to the next argument if successful.
- * If no equation number or range is given, assume the current equation is wanted.
+ * If no equation number or range is given, assume the current equation is wanted and
+ * don't skip anything.
  *
  * Return true if successful,
  * with the starting equation number in "*ip"
@@ -887,17 +939,22 @@ int	*ip, *jp;
 		} else {
 			*ip = cur_equation;
 		}
-		if (*ip < 0 || *ip >= N_EQUATIONS) {
-			error(_("Invalid equation number."));
-			return false;
-		}
 		if (*cp != '-') {
-			if (not_defined(*ip)) {
-				return false;
+			if (*cp == '\0' || isspace(*cp)) {
+				if (not_defined(*ip)) {
+					return false;
+				}
+				*jp = *ip;
+				*cpp = skip_space(cp);
+				return true;
+			} else {
+use_current:
+				if (current_not_defined()) {
+					return false;
+				}
+				*jp = *ip = cur_equation;
+				return true;
 			}
-			*jp = *ip;
-			*cpp = skip_space(cp);
-			return true;
 		}
 		(cp)++;
 		if (isdigit(*cp)) {
@@ -905,7 +962,10 @@ int	*ip, *jp;
 		} else {
 			*jp = cur_equation;
 		}
-		if (*jp < 0 || *jp >= N_EQUATIONS) {
+		if (*cp && !isspace(*cp)) {
+			goto use_current;
+		}
+		if (*ip < 0 || *ip >= N_EQUATIONS || *jp < 0 || *jp >= N_EQUATIONS) {
 			error(_("Invalid equation number."));
 			return false;
 		}
