@@ -1,59 +1,39 @@
 /*
  * Main include file for Mathomatic.  Can be edited.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
-#ifndef M_EULER
-#define M_EULER	0.57721566490153286060651209008      /* Euler constant (from GSL) */
-#endif 
-#ifndef	M_PI
-#define M_PI	3.14159265358979323846	/* pi */
-#endif
-#ifndef	M_E
-#define M_E	2.7182818284590452354	/* e */
-#endif
-
-#ifndef	max
-#define max(a, b)    (((a) > (b)) ? (a) : (b))		/* return the maximum of two values */
-#endif
-#ifndef	min
-#define min(a, b)    (((a) < (b)) ? (a) : (b))		/* return the minimum of two values */
-#endif
-
-#ifndef	INFINITY
-#define INFINITY	HUGE_VAL			/* the floating point, positive infinity constant */
-#endif
-
+#if	sun
+#define	INFINITY_NAME	"Infinity"	/* set this to the name of the infinity constant as displayed by printf(3) */
+#define NAN_NAME	"NaN"		/* set this to the name of the NaN constant as displayed by printf(3) */
+#else
 #define	INFINITY_NAME	"inf"	/* set this to the name of the infinity constant as displayed by printf(3) */
 #define NAN_NAME	"nan"	/* set this to the name of the NaN constant as displayed by printf(3) */
-
-#ifndef	isfinite
-#define	isfinite(d)	finite(d)			/* true if d is finite (not infinity nor NaN) */
 #endif
 
-#ifndef PATH_MAX					/* make sure the maximum pathname length is set */
+#ifndef PATH_MAX					/* make sure the maximum file pathname length is set */
 #define PATH_MAX	4096
 #endif
 
-#define	ARR_CNT(a)	((int) (sizeof(a)/sizeof(a[0])))	/* returns the number of elements in an array */
-#define CLEAR_ARRAY(a)	memset(a, 0, sizeof(a))			/* sets all elements of an array to zero */
-
 #define	MAX_K_INTEGER	1.0e15		/* maximum safely representable floating point integer, 15 digits for doubles */
 
 #define	blt(dst, src, cnt)	memmove((char *) (dst), (char *) (src), (size_t) (cnt))	/* memory copy function; must allow overlapping of src and dst */
@@ -68,13 +48,15 @@
 
 #define	STANDARD_SCREEN_COLUMNS	80			/* default number of columns of characters on the screen */
 #define	STANDARD_SCREEN_ROWS	24			/* default number of lines on the screen */
+#define	TEXT_ROWS	STANDARD_SCREEN_ROWS		/* number of lines per page in the symbolic math library */
+#define	TEXT_COLUMNS	STANDARD_SCREEN_COLUMNS		/* number of columns per page in the symbolic math library */
 
 #define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temporary file template for mkstemp(3) */
 
 #define	PROMPT_STR	"-> "				/* user interface main prompt strings, preceded by the current equation number */
-#define	HTML_PROMPT_STR	"&mdash;&gt; "			/* main prompt in HTML output mode, should be same number of columns as above */
+#define	HTML_PROMPT_STR	"&minus;&gt; "			/* main prompt in HTML output mode, should be same number of columns as above */
 
-#define	MAX_CMD_LEN	(max(PATH_MAX, 512))		/* maximum command line length (not expression input), also max filename length */
+#define	MAX_CMD_LEN	min((max(PATH_MAX, 512)), 32768)	/* maximum command line length (not expression input), also max filename length */
 #define	MAX_PROMPT_LEN	STANDARD_SCREEN_COLUMNS		/* maximum length of prompts */
 
 /*
@@ -104,13 +86,13 @@ typedef union {
 /* Valid operators follow (in precedence order), 0 is reserved for no operator: */
 #define	PLUS		1	/* a + b */
 #define	MINUS		2	/* a - b */
-#define	TIMES		3	/* a * b */
-#define	DIVIDE		4	/* a / b */
-#define	MODULUS		5	/* a % b */
-#define IDIVIDE		6	/* a // b */
-#define	POWER		7	/* a ^ b */
-#define	FACTORIAL	8	/* a! */
-#define	NEGATE		9	/* special parser operator not seen outside parser */
+#define	NEGATE		3	/* special parser operator not seen outside parser */
+#define	TIMES		4	/* a * b */
+#define	DIVIDE		5	/* a / b */
+#define	MODULUS		6	/* a % b */
+#define IDIVIDE		7	/* a // b */
+#define	POWER		8	/* a ^ b */
+#define	FACTORIAL	9	/* a! */
 } storage_type;
 
 typedef	struct {		/* storage structure for each token in an expression */
@@ -125,12 +107,8 @@ typedef	struct {		/* storage structure for each token in an expression */
  * This affects maximum memory usage.
  */
 #ifndef	N_EQUATIONS
-#if	HANDHELD
-#define	N_EQUATIONS	50
-#else
 #define	N_EQUATIONS	200
 #endif
-#endif
 
 /*
  * The following defines the default maximum mathematical expression size.
@@ -151,7 +129,7 @@ typedef	struct {		/* storage structure for each token in an expression */
 #error DEFAULT_N_TOKENS out of range!
 #endif
 
-#define	DIVISOR_SIZE		(DEFAULT_N_TOKENS / 2)	/* a nice maximum divisor size */
+#define	DIVISOR_SIZE		min((DEFAULT_N_TOKENS / 2), 15000)	/* a nice maximum divisor size */
 
 /*
  * All Mathomatic variables are referenced by the value in a C long int variable.
