@@ -1,27 +1,30 @@
 /*
  * Mathomatic symbolic differentiation routines and related commands.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-static int	d_recurse();
+static int d_recurse(token_type *equation, int *np, int loc, int level, long v);
 
 /*
  * Compute the derivative of an equation side, with respect to variable "v",
@@ -357,36 +360,36 @@ char	*cp;
 		if (extra_characters(cp))
 			return false;
 	}
+	show_usage = false;
 	if (no_vars(source, *nps, &v)) {
-#if	!SILENT
-		error(_("Current expression contains no variables; the derivative would be zero."));
-		return false;
-#endif
-	}
-	if (v == 0) {
+		warning(_("Current expression contains no variables; the derivative will be zero."));
+	} else if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
 		}
 	}
-#if	!SILENT
-	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		error(_("Variable not found; the derivative would be zero."));
+	if (v == 0) {
+		error(_("No differentiation variable specified."));
 		return false;
 	}
+	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
+		warning(_("Variable not found; the derivative will be zero."));
+	}
+#if	!SILENT
 	if (debug_level >= 0) {
 		list_var(v, 0);
 		if (n_rhs[cur_equation]) {
-			printf(_("Differentiating the RHS with respect to (%s)"), var_str);
+			fprintf(gfp, _("Differentiating the RHS with respect to (%s)"), var_str);
 		} else {
-			printf(_("Differentiating with respect to (%s)"), var_str);
+			fprintf(gfp, _("Differentiating with respect to (%s)"), var_str);
 		}
 		if (order != 1) {
-			printf(_(" %ld times"), order);
+			fprintf(gfp, _(" %ld times"), order);
 		}
 		if (simplify_flag) {
-			printf(_(" and simplifying"));
+			fprintf(gfp, _(" and simplifying"));
 		}
-		printf("...\n");
+		fprintf(gfp, "...\n");
 	}
 #endif
 	blt(dest, source, *nps * sizeof(token_type));
@@ -466,6 +469,7 @@ char	*cp;
 		if (extra_characters(cp))
 			return false;
 	}
+	show_usage = false;
 	if (no_vars(source, n, &v)) {
 		error(_("Current expression contains no variables."));
 		return false;
@@ -544,6 +548,7 @@ char	*cp;
 	n_lhs[i] = 0;
         if (our < 0) {
                 error(_("Out of free equation spaces."));
+		show_usage = false;
 		return false;
 	}
 	if (n_rhs[cur_equation]) {
@@ -571,6 +576,7 @@ char	*cp;
 		}
 		cp = cp1;
 	}
+	show_usage = false;
 	no_vars(source, *nps, &v);
 	if (v == 0) {
 		if (!prompt_var(&v)) {
@@ -592,6 +598,7 @@ char	*cp;
 		input_column += (cp - cp_start);
 		cp = parse_expr(lhs[our], &our_nlhs, cp, true);
 		if (cp == NULL || extra_characters(cp) || our_nlhs <= 0) {
+			show_usage = true;
 			return false;
 		}
 	} else {
@@ -622,14 +629,14 @@ char	*cp;
 		}
 	}
 #if	!SILENT
-	printf(_("Computing the Taylor series"));
+	fprintf(gfp, _("Computing the Taylor series"));
 	if (n_rhs[cur_equation]) {
-		printf(_(" of the RHS"));
+		fprintf(gfp, _(" of the RHS"));
 	}
 	if (simplify_flag) {
-		printf(_(" and simplifying"));
+		fprintf(gfp, _(" and simplifying"));
 	}
-	printf("...\n");
+	fprintf(gfp, "...\n");
 #endif
 	n = 0;
 	i1 = 0;
@@ -729,7 +736,7 @@ loop_again:
 		}
 	}
 #if	!SILENT
-	printf(_("%ld derivative%s applied.\n"), n, (n == 1) ? "" : "s");
+	fprintf(gfp, _("%ld derivative%s applied.\n"), n, (n == 1) ? "" : "s");
 #endif
 	if (n_rhs[cur_equation]) {
 		n_lhs[i] = n_lhs[cur_equation];
@@ -763,7 +770,7 @@ char	*cp;
 		n_lhs[cur_equation] = 1;
 		lhs[cur_equation][0].level = 1;
 		lhs[cur_equation][0].kind = VARIABLE;
-		parse_var(&lhs[cur_equation][0].token.variable, "answer");
+		parse_var(&lhs[cur_equation][0].token.variable, "limit");
 	}
 	if (!solved_equation(cur_equation)) {
 		error(_("The current equation is not solved for a variable."));
@@ -777,6 +784,7 @@ char	*cp;
 			return false;
 		}
 	}
+	show_usage = false;
 	if (no_vars(rhs[cur_equation], n_rhs[cur_equation], &v)) {
 		error(_("Current expression contains no variables."));
 		return false;
@@ -797,6 +805,7 @@ char	*cp;
 		input_column += (cp - cp_start);
 		cp = parse_expr(tes, &n_tes, cp, true);
 		if (cp == NULL || extra_characters(cp) || n_tes <= 0) {
+			show_usage = true;
 			return false;
 		}
 	} else {
@@ -806,12 +815,20 @@ char	*cp;
 			return false;
 		}
 	}
+	simp_loop(tes, &n_tes);
+#if	!SILENT
+	if (debug_level >= 0) {
+		list_var(v, 0);
+		fprintf(gfp, _("Taking the limit as %s goes to "), var_str);
+		list_proc(tes, n_tes, false);
+		fprintf(gfp, "\n");
+	}
+#endif
 /* copy the current equation to a new equation space, then simplify and work on the copy: */
 	copy_espace(cur_equation, i);
 	simpa_side(rhs[i], &n_rhs[i], false, false);
 
 /* see if the limit expression is positive infinity: */
-	simp_loop(tes, &n_tes);
 	if (n_tes == 1 && tes[0].kind == CONSTANT && tes[0].token.constant == INFINITY) {
 /* To take the limit to positive infinity, */
 /* replace infinity with zero and replace the limit variable with its reciprocal: */
