@@ -1,21 +1,24 @@
 /*
  * Mathomatic global variable extern definitions, from file "globals.c".
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -62,7 +65,9 @@ extern char		plot_prefix[256];
 extern int		integer_coefficients;
 extern int		right_associative_power;
 extern int		power_starstar;
+#if	!SILENT
 extern int		debug_level;
+#endif
 extern int		domain_check;
 extern int		color_flag;
 extern int		bold_colors;
@@ -82,12 +87,18 @@ extern char		*prog_name;
 extern char		*var_names[MAX_VAR_NAMES];
 extern char		var_str[MAX_VAR_LEN+80];
 extern char		prompt_str[MAX_PROMPT_LEN];
-extern char		rc_file[PATH_MAX];
+#if	!SECURE && !LIBRARY
+extern char		rc_file[MAX_CMD_LEN];
+#endif
 
+#if	CYGWIN || MINGW
 extern char		*dir_path;
+#endif
+#if	READLINE
 extern char		*last_history_string;
 extern char		*history_filename;
-extern char		history_filename_storage[PATH_MAX];
+extern char		history_filename_storage[MAX_CMD_LEN];
+#endif
 
 extern double		unique[];
 extern int		ucnt[];
@@ -96,6 +107,7 @@ extern int		uno;
 extern sign_array_type	sign_array;
 extern FILE		*default_out;
 extern FILE		*gfp;
+extern char		*gfp_filename;
 extern jmp_buf		jmp_save;
 extern int		eoption;
 extern int		test_mode;
@@ -104,10 +116,16 @@ extern int		quiet_mode;
 extern int		echo_input;
 extern volatile int	abort_flag;
 extern int		pull_number;
+#if	!SECURE
 extern int		security_level;
+#endif
 extern int		repeat_flag;
+extern int		show_usage;
 
 extern char		*result_str;
 extern int		result_en;
 extern const char	*error_str;
 extern const char	*warning_str;
+#if	LIBRARY
+extern char vscreen[TEXT_ROWS][TEXT_COLUMNS];
+#endif
