@@ -1,33 +1,36 @@
 /*
  * Mathomatic symbolic factorizing routines, not polynomial factoring.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-static int	fplus_recurse();
-static int	fplus_sub();
-static int	big_fplus();
-static int	ftimes_recurse();
-static int	ftimes_sub();
-static int	fpower_recurse();
-static int	fpower_sub();
+static int fplus_recurse(token_type *equation, int *np, int loc, int level, long v, double d, int whole_flag, int div_only);
+static int fplus_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level, long v, double d, int whole_flag, int div_only);
+static int big_fplus(token_type *equation, int level, int diff_sign, int sop1, int op1, int op2, int i1, int i2, int b1, int b2, int ai, int aj, int i, int j, int e1, int e2);
+static int ftimes_recurse(token_type *equation, int *np, int loc, int level);
+static int ftimes_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level);
+static int fpower_recurse(token_type *equation, int *np, int loc, int level);
+static int fpower_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level);
 
 /*
  * Factor divides only: (a/c + b/c) -> ((a+b)/c).
@@ -68,6 +71,12 @@ int		*np;
  * If "v" is a variable, only factor expressions containing that variable, with no Horner factoring.
  * If "d" is not equal to 0.0 or 1.0, factor only expressions containing "v" raised to the power of "d".
  *
+ * In order to factor down to the smallest possible expression,
+ * factoring the most common and largest sub-expressions needs to be done first.
+ * Mathomatic does not have the ability to do that.
+ * However, simpb_side() factors expressions with the most used variables first,
+ * which is helpful.
+ *
  * Return true if equation side was modified.
  */
 int
@@ -84,7 +93,7 @@ double		d;		/* control exponent */
  * Recursively factor at "level" of parentheses and deeper,
  * beginning at index "loc".
  *
- * Return true if the expression was modified.
+ * Return true if equation side was modified.
  */
 static int
 fplus_recurse(equation, np, loc, level, v, d, whole_flag, div_only)
@@ -520,7 +529,7 @@ end_mess:
 			}
 			if (power < 1.0)
 				goto f_inner;
-			modf(power, &power);
+			modf(power, &power);	/* same as power = trunc(power); ? */
 			goto horner_factor;
 		}
 		d1 = i - ai;
@@ -766,9 +775,15 @@ int		*np, loc, i1, n1, i2, n2, level;
 	} else {
 		op1 = equation[i1-1].token.operatr;
 	}
+#if	1
 	if ((n1 == 1 && equation[i1].kind == CONSTANT) && (n2 == 1 && equation[i2].kind == CONSTANT)) {
 		return false;
 	}
+#else
+	if ((n1 == 1 && equation[i1].kind == CONSTANT) || (n2 == 1 && equation[i2].kind == CONSTANT)) {
+		return false;
+	}
+#endif
 	both_divide = (op1 == DIVIDE && op2 == DIVIDE);
 	if (se_compare(&equation[i1], n1, &equation[i2], n2, &diff_sign)) {
 		i = e1;
