@@ -1,31 +1,34 @@
 /*
  * Mathomatic floating point constant factorizing routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-static void	try_factor();
-static int	fc_recurse();
+static void try_factor(double arg);
+static int fc_recurse(token_type *equation, int *np, int loc, int level, int level_code);
 
 /* The following data is used to factor integers: */
-static double nn, vv;
+static double nn, sqrt_value;
 static double skip_multiples[] = {	/* Additive array that skips over multiples of 2, 3, 5, and 7. */
 	10, 2, 4, 2, 4, 6, 2, 6,
 	 4, 2, 4, 6, 6, 2, 6, 4,
@@ -61,22 +64,28 @@ double	value;
 		/* not an integer */
 		return false;
 	}
-	vv = 1.0 + sqrt(fabs(nn));
+	sqrt_value = 1.0 + sqrt(fabs(nn));
 	try_factor(2.0);
 	try_factor(3.0);
 	try_factor(5.0);
 	try_factor(7.0);
 	d = 1.0;
-	while (d <= vv) {
+	while (d <= sqrt_value) {
 		for (i = 0; i < ARR_CNT(skip_multiples); i++) {
 			d += skip_multiples[i];
 			try_factor(d);
 		}
 	}
 	if (nn != 1.0) {
+		if (nn < 0 && nn != -1.0) {
+			try_factor(fabs(nn));
+		}
 		try_factor(nn);
 	}
-	if (value != multiply_out_unique()) {
+	if (uno == 0) {
+		try_factor(1.0);
+	}
+	if (nn != 1.0 || value != multiply_out_unique()) {
 		error_bug("Internal error factoring integers.");
 	}
 	return true;
@@ -91,16 +100,20 @@ try_factor(arg)
 double	arg;
 {
 	while (fmod(nn, arg) == 0.0) {
-		if (uno > 0 && unique[uno-1] == arg) {
+		if (uno > 0 && ucnt[uno-1] > 0 && unique[uno-1] == arg) {
 			ucnt[uno-1]++;
 		} else {
+			while (uno > 0 && ucnt[uno-1] <= 0)
+				uno--;
 			unique[uno] = arg;
-			ucnt[uno] = 1;
-			uno++;
+			ucnt[uno++] = 1;
 		}
 		nn /= arg;
-		vv = 1.0 + sqrt(fabs(nn));
-		if (nn <= 1.0 && nn >= -1.0)
+		if (fmod(nn, 1.0) != 0.0) {
+			error_bug("Turned non-integer in try_factor().");
+		}
+		sqrt_value = 1.0 + sqrt(fabs(nn));
+		if (fabs(nn) <= 1.5 || fabs(arg) <= 1.5)
 			break;
 	}
 }
@@ -170,9 +183,9 @@ is_prime()
 }
 
 /*
- * Factor integers in an equation side.
+ * Factor integers into their prime factors in an equation side.
  *
- * Return true if equation side was modified.
+ * Return true if the equation side was modified.
  */
 int
 factor_int(equation, np)
