@@ -3,21 +3,24 @@
  *
  * Everything that depends on the command table goes here.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -49,46 +52,47 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces.", "Related command: calculate" },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"factor\"] [variable number-of-iterations]",		"Temporarily plug in values for variables and approximate.", "This command may be preceded with \"repeat\"." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"factor\"] [feedback-variable number-of-iterations]",	"Temporarily plug in values for variables and approximate.", "This command may be preceded with \"repeat\"." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to quickly restart Mathomatic." },
 {	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables" },
-{	"compare",	NULL,			compare_cmd,		"[\"symbolic\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if mathematically the same." },
+{	"compare",	NULL,			compare_cmd,		"[\"symbolic\" \"approx\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if mathematically the same." },
 {	"copy",		NULL,			copy_cmd,		"[equation-number-ranges]",				"Duplicate the contents of the specified equation spaces." },
-{	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] [variable or \"all\"] [order]",	"Symbolically differentiate and simplify, order times." },
-{	"display",	NULL,			display_cmd,		"[\"factor\"] [equation-number-ranges]",			"Display equation spaces in pretty multi-line (2D) fraction format." },
+{	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] variable or \"all\" [order]",		"Symbolically differentiate and simplify, order times." },
+{	"display",	NULL,			display_cmd,		"[\"factor\" \"mixed\"] [equation-number-ranges]",	"Display equation spaces in pretty, 2D multi-line fraction format." },
 {	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD.", "This command may be preceded with \"repeat\"." },
 {	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
-#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
-{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in." },
+#if	SHELL_OUT
+{	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in.", "Editor name in EDITOR environment variable." },
 #endif
 {	"eliminate",	NULL,			eliminate_cmd,		"variables or \"all\" [\"using\" equation-number]",	"Substitute the specified variables with solved equations.", "This command may be preceded with \"repeat\"." },
 {	"extrema",	NULL,			extrema_cmd,		"[variable] [order]",					"Show where the slope of the current equation equals zero.", "Helps with finding the minimums and maximums." },
 {	"factor",	"collect",		factor_cmd,		"[\"number\" [integers]] or [\"power\"] [equation-number-range] [variables]",	"Factor variables in equation spaces or factor given integers." },
-{	"fraction",	"together",		fraction_cmd,		"[equation-number-range]",				"Convert expression to a single simple fraction." },
+{	"for",		NULL,			for_cmd,		"variable start end [step-size]",			"Evaluate and display the current expression for each value of variable.", "Same syntax as the sum and product commands." },
+{	"fraction",	NULL,			fraction_cmd,		"[\"numerator\" \"denominator\"] [equation-number-range]",	"Convert expression to a single simple algebraic fraction.", "This command may be preceded with \"repeat\"." },
 #if	HELP
-{	"help",		NULL,			help_cmd,		"[topics or command-names]",				"Short, built-in help and reference." },
+{	"help",		"?",			help_cmd,		"[topics or command-names]",				"Short, built-in help and reference." },
 #endif
 {	"imaginary",	NULL,			imaginary_cmd,		"[variable]",						"Copy the imaginary part of the current expression.", "Related command: real" },
 {	"integrate",	"integral",		integrate_cmd,		"[\"constant\" or \"definite\"] variable [order]",	"Symbolically integrate polynomials order times, then simplify." },
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials.", "This command only works with polynomials." },
-{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression (experimental).", "This command is experimental." },
-{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\" or \"hexadecimal\"] [equation-number-ranges]",	"Display equation spaces in single-line format.", "Options to export to other math programs." },
+{	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression.", "This limit command is experimental." },
+{	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\" or \"hex\"] [equation-number-ranges]",	"Display equation spaces in single-line (one-dimensional) format.", "Options to export expressions to other math programs." },
 {	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule.", "This command cannot integrate over singularities." },
-{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations.", "Related command: code" },
+{	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations." },
 {	"pause",	NULL,			pause_cmd,		"[text]",						"Display a line of text and wait for user to press the Enter key." },
-#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
-{	"plot",		NULL,			plot_cmd,		"[x-range [y-range]] [expressions,]",			"Automatically plot expressions in 2D or 3D with Gnuplot.", "Plots variable x, and y if 3D surface plot." },
+#if	SHELL_OUT
+{	"plot",		NULL,			plot_cmd,		"[equation-number-ranges] [x-range [y-range]] [expressions,]",	"Automatically plot expressions in 2D or 3D with gnuplot.", "Plots variable x; if expression contains y, do a 3D surface plot." },
 #endif
 {	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum" },
 #if	READLINE
-{	"push",		NULL,			push_cmd,		"[equation-number-range]",				"Push equation spaces into the readline history for editing.", "Available only if readline is enabled." },
+{	"push",		NULL,			push_cmd,		"[equation-number-ranges]",				"Push equation spaces into the readline history for editing.", "Available only if readline is enabled." },
 #endif
 #if	!LIBRARY
 {	"quit",		"exit",			quit_cmd,		"[exit-value]",						"Terminate this program without saving." },
 #endif
 #if	!SECURE
-{	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in.", "Related command: save" },
+{	"read",		NULL,			read_cmd,		"file-name",						"Read in a text file as if it was typed in.", "This command may be preceded with \"repeat\"." },
 #endif
 {	"real",		NULL,			real_cmd,		"[variable]",						"Copy the real part of the current expression.", "Related command: imaginary" },
 {	"replace",	NULL,			replace_cmd,		"[variables [\"with\" expression]]",			"Substitute variables in the current equation with expressions." },
@@ -96,37 +100,38 @@ static com_type com_list[] = {
 #if	!SECURE
 {	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read" },
 #endif
-{	"set",		NULL,			set_cmd,		"[[\"no\"] option] ...",				"Display, set, or save current session options.", "\"set\" by itself will show all current option settings." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"sign\"] [\"symbolic\"] [\"quick[est]\"] [\"fraction\"] [equation-number-range]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
-{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [\"for\"] variable or \"0\"",		"Solve the current equation for a variable or for zero.", "The verify option checks all displayed solutions." },
+{	"set",		NULL,			set_cmd,		"[[\"no\"] option [value]] ...",			"Display, set, or save current session options.", "\"set\" by itself will show all current option settings." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\" \"symbolic\" \"quick[est]\" \"fraction\"] [equation-number-range]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
+{	"solve",	NULL,			solve_cmd,		"[\"verify\"] [equation-number-range] [\"for\"] variable or \"0\"",	"Solve the specified equations for a variable or for zero.", "The verify option checks all displayed solutions." },
 {	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product" },
 #if	!LIBRARY
-{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, show total and optionally the average." },
+{	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, showing running total and average." },
 #endif
 {	"taylor",	NULL,			taylor_cmd,		"[\"nosimplify\"] variable order point",		"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,		"[\"fraction\"] [\"quick\"] [\"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
-{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-range]",	"Show all variable names used within the specified expressions.", "Related command: code" },
-{	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version and license information." }
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"fraction\" \"quick\" \"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
+{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-ranges]",	"Show all variable names used within the specified expressions.", "Related command: code" },
+{	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version, compiler, and license information." }
 };
 
 #if	HELP && !LIBRARY
 char	*example_strings[] = {
 	"; The derivative of the absolute value function:\n",
 	"|x|\n",
-	"derivative\n",
+	"derivative ; The result gives the sign of x:\n",
 	"pause\n",
 	"\n",
-	"; Enter a common temperature conversion formula:\n",
+	"; A common temperature conversion formula:\n",
 	"fahrenheit = (9*celsius/5) + 32\n",
 	"repeat calculate ; plug in values until an empty line is entered\n",
 	"; Solve for the other variable and simplify the result:\n",
 	"solve for celsius\n",
 	"simplify\n",
+	"repeat calculate ; plug in values until an empty line is entered\n",
 	"variables ; show all variables that occur in expressions\n",
 	"pause\n",
 	"\n",
 	"; Expand the following into polynomial form, then refactor:\n",
-	"(x+1)^5\n",
+	"(x+1)^6\n",
 	"unfactor\n",
 	"simplify\n",
 	NULL
@@ -177,9 +182,14 @@ process_parse(n, cp)
 int	n;
 char	*cp;
 {
-	int	i;
-	char	*cp1, *ep;
-	int	equals_flag = false;
+	int		i;
+	char		*cp1, *ep;
+	int		equals_flag = false;
+	int		rv;
+#if	!LIBRARY
+	static int	last_autocalc_en = -1;
+	long		answer_v = 0;		/* Mathomatic answer variable */
+#endif
 
 	if ((ep = parse_equation(n, cp))) {
 		for (cp1 = cp; cp1 < ep; cp1++) {
@@ -208,10 +218,14 @@ char	*cp;
 				    || (lhs[n][0].kind == VARIABLE && ((lhs[n][0].token.variable & VAR_MASK) > SIGN || equals_flag))))
 				    || (n_rhs[n] == 1 && ((rhs[n][0].kind == CONSTANT && rhs[n][0].token.constant == 0.0)
 				    || rhs[n][0].kind == VARIABLE))) {
-					if (solve_espace(n, cur_equation)) {
+					rv = solve_espace(n, cur_equation);
+					n_lhs[n] = 0;
+					n_rhs[n] = 0;
+					if (rv) {
 						return return_result(cur_equation);
+					} else {
+						return false;
 					}
-					return false;
 				}
 			}
 			if (n_rhs[n]) {
@@ -242,17 +256,24 @@ char	*cp;
 				/* make the expression an equation by making the LHS the "answer" variable */
 				lhs[n][0].level = 1;
 				lhs[n][0].kind = VARIABLE;
-				parse_var(&lhs[n][0].token.variable, "answer");
+				parse_var(&answer_v, "answer");	/* convert to a Mathomatic variable */
+				lhs[n][0].token.variable = answer_v;
 				n_lhs[n] = 1;
-				i = cur_equation;
-				/* temporarily make it the current equation and run the calculate command on it */
+				/* make it the current equation and run the calculate command on it */
 				cur_equation = n;
-				calculate_cmd("");
-				cur_equation = i;
-				/* delete from memory */
-				n_lhs[n] = 0;
-				n_rhs[n] = 0;
-				return true;
+				rv = calculate_cmd("");	/* display the approximation */
+				/* Keep the current input until next autocalc */
+				i = last_autocalc_en;
+				if (i >= 0 && i < n_equations) {
+					if (i != n && n_lhs[i] == 1 && lhs[i][0].kind == VARIABLE
+					    && lhs[i][0].token.variable == answer_v) {
+						/* delete previous answer from memory */
+						n_lhs[i] = 0;
+						n_rhs[i] = 0;
+					}
+				}
+				last_autocalc_en = n;
+				return rv;
 			}
 #endif
 		}
@@ -267,22 +288,40 @@ return_ok:
  * Run a line of main prompt input to Mathomatic.
  * It may be a command, an expression or equation to store, etc.
  *
- * Return true if successful.
+ * Return true if line starts with a colon (:) or if successful.
  */
 int
 process(cp)
 char	*cp;
+{
+	if (cp && cp[0] == ':') {
+		input_column++;
+		process_rv(cp + 1);
+		return true;
+	} else {
+		return process_rv(cp);
+	}
+}
+
+/*
+ * Run a line of main prompt input to Mathomatic.
+ * It may be a command, an expression or equation to store, etc.
+ *
+ * Return true if successful.
+ */
+int
+process_rv(cp)
+char	*cp;
 {
 	char	*cp1;
 	char	*cp_start;
 	int	i;
 	int	len;
 	int	rv;
-	char	buf2[MAX_CMD_LEN];
+	char	buf2[MAX_CMD_LEN];	/* do not make this static! */
 	int	our_repeat_flag = false;
 #if	!SECURE
 	FILE	*fp;
-	char	*filename;
 	int	append_flag;
 	int	i1;
 #endif
@@ -300,44 +339,55 @@ char	*cp;
 		switch (*cp) {
 		case '+':
 		case '-':
-			i = decstrtol(cp, &cp1);
+			i = strtol(cp, &cp1, 10);
 			i = cur_equation + i;
 			break;
 		default:
-			i = decstrtol(cp, &cp1) - 1;
+			i = strtol(cp, &cp1, 10) - 1;
 			break;
 		}
 		if (cp == cp1) {
 			return true;	/* treat as comment */
 		}
-		if (!alloc_to_espace(i)) {
-			put_up_arrow(cp - cp_start, _("Equation number out of range."));
-			return false;
+		if (*cp1 == '\0' || *cp1 == ':' || isspace(*cp1)) {
+			if (!alloc_to_espace(i)) {
+				put_up_arrow(cp - cp_start, _("Equation number out of range."));
+				return false;
+			}
+			cp = cp1;
+			if (*cp == ':') {
+				cp++;
+			}
+			cp = skip_space(cp);
+			if (*cp) {
+				input_column += (cp - cp_start);
+				return parse(i, cp);
+			}
+			cur_equation = i;
+			return_result(cur_equation);
+			return true;
+		} else {
+			cp--;
 		}
-		cp = cp1;
-		if (*cp == ':') {
-			cp++;
-		}
-		cp = skip_space(cp);
-		if (*cp) {
-			input_column += (cp - cp_start);
-			return parse(i, cp);
-		}
-		cur_equation = i;
-		return_result(cur_equation);
-		return true;
 	}
-#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+#if	SHELL_OUT
 /* handle shell escape */
 	if (security_level == 0 && *cp == '!') {
 		cp1 = getenv("SHELL");
 		if (cp1 == NULL) {
+#if	MINGW
+			cp1 = "cmd";
+#else
 			cp1 = "/bin/sh";
+#endif
 		}
+#if	0
 		if (access(cp1, X_OK)) {
+			perror(cp1);
 			error(_("Shell not found or not executable, check SHELL environment variable."));
 			return false;
 		}
+#endif
 		cp = skip_space(cp + 1);
 		rv = shell_out(*cp ? cp : cp1);
 		return !rv;
@@ -382,10 +432,10 @@ do_repeat:
 			if (security_level < 2) {
 				/* handle output redirection */
 				append_flag = false;
-				filename = NULL;
+				gfp_filename = NULL;
 				for (i1 = strlen(buf2) - 1; i1 >= 0; i1--) {
 					if (buf2[i1] == '>') {
-						filename = skip_space(&buf2[i1+1]);
+						gfp_filename = skip_space(&buf2[i1+1]);
 						if (i1 && buf2[i1-1] == '>') {
 							i1--;
 							append_flag = true;
@@ -394,15 +444,16 @@ do_repeat:
 						break;
 					}
 				}
-				if (filename) {
+				if (gfp_filename) {
 					if (append_flag) {
-						fp = fopen(filename, "a");
+						fp = fopen(gfp_filename, "a");
 					} else {
-						fp = fopen(filename, "w");
+						fp = fopen(gfp_filename, "w");
 					}
 					if (fp == NULL) {
-						perror(filename);
+						perror(gfp_filename);
 						error(_("Can't open redirected output file for writing."));
+						gfp_filename = NULL;
 						return false;
 					}
 					if (gfp != stdout && gfp != stderr && gfp != default_out) {
@@ -415,6 +466,7 @@ do_repeat:
 			remove_trailing_spaces(buf2);
 			pull_number = 1;
 /* execute the command by calling the command function */
+			show_usage = true;
 			repeat_flag = our_repeat_flag;
 			rv = (*com_list[i].func)(buf2);
 			repeat_flag = false;
@@ -424,9 +476,10 @@ do_repeat:
 					fclose(gfp);
 				gfp = default_out;
 			}
+			gfp_filename = NULL;
 #endif
 #if	!SILENT
-			if (!rv && debug_level >= 0) {
+			if (show_usage && !rv && debug_level >= 0) {
 				printf("Command usage: %s %s\n", com_list[i].name, com_list[i].usage);
 			}
 #endif
@@ -434,7 +487,7 @@ do_repeat:
 		}
 	}
 	if (our_repeat_flag) {
-		error(_("Follow \"repeat\" with command to automatically repeat."));
+		error(_("Follow \"repeat\" with a command to automatically repeat."));
 		return false;
 	}
 /* cp is not a command, so parse the expression */
@@ -467,8 +520,9 @@ char	*cp;	/* string to process, will be modified */
 #if	LIBRARY
 		fprintf(gfp, "%d%s%s", cur_equation + 1, PROMPT_STR, cp);
 #else
-		default_color();
+		set_color(3);		/* blue prompt */
 		input_column = printf("%d%s", cur_equation + 1, html_flag ? HTML_PROMPT_STR : PROMPT_STR);
+		default_color();
 		if (html_flag) {
 			input_column -= (strlen(HTML_PROMPT_STR) - strlen(PROMPT_STR));
 			printf("<b>%s</b>", cp);	/* make input bold */
@@ -488,20 +542,21 @@ char	*cp;	/* string to process, will be modified */
 	return process(cp);
 }
 
-#if	(UNIX || CYGWIN) && !SECURE && !LIBRARY
+#if	SHELL_OUT
 /*
- * Execute a system command.  Note that system(3) requires "/bin/sh".
+ * Execute a shell command.  Note that system(3) requires "/bin/sh".
  *
- * Returns exit value of command (0 if no error).
+ * Returns exit status of command (0 if no error).
  */
 int
 shell_out(cp)
-char	*cp;	/* system command string */
+char	*cp;	/* shell command string */
 {
 	int	rv;
 
-	if (security_level) {
-		error_bug("Shelling out disabled.");
+	if (security_level > 0) {
+		error(_("Shelling out disabled by security level."));
+		return -1;
 	}
 	reset_attr();
 
@@ -613,13 +668,16 @@ char	*cp;
 	int	row;
 
 	if (*cp == '\0') {
+intro:
 		/* default help text: */
 		P("Mathomatic is a Computer Algebra System (CAS) and calculator program.");
+#if	!LIBRARY
 		P("To see helpful interactive examples, type \"help examples\".");
+#endif
 		P("Type \"help equations\" for help with entering expressions and equations.");
 		P("Type \"help all\" for a summary and syntax of all commands.");
 		P("Type \"help usage\" to display the syntax of all commands.");
-		P("Other help keywords: constants, geometry, license, bugs, options.");
+		P("Other help keywords: constants, color, license, bugs, geometry.");
 		P("\"help\" or \"?\" followed by a command name will give info on that command.");
 		fprintf(gfp, "These are the %d commands for this version of Mathomatic:\n", ARR_CNT(com_list));
 		for (i = 0; i < ARR_CNT(com_list); i++) {
@@ -632,6 +690,22 @@ char	*cp;
 
 		P("\n\nTo select an equation space, type the equation number at the main prompt.");
 		P("To solve the current equation, type the variable name at the main prompt.");
+		P("For more help, go to the website: www.mathomatic.org");
+	} else if (strcasecmp(cp, "table") == 0) {
+		fprintf(gfp, "Mathomatic version %s Quick Reference Card\n", VERSION);
+		fprintf(gfp, "Command\tUsage\tNotes\n");
+		for (i = 0; i < ARR_CNT(com_list); i++) {
+			fprintf(gfp, "%s", com_list[i].name);
+			fprintf(gfp, "\t%s %s", com_list[i].name, com_list[i].usage);
+			if (com_list[i].secondary_name) {
+				fprintf(gfp, "\tAlternate name for this command: %s", com_list[i].secondary_name);
+			} else if (com_list[i].extra) {
+				fprintf(gfp, "\t%s", com_list[i].extra);
+			} else {
+				fprintf(gfp, "\t%s", com_list[i].info);
+			}
+			fprintf(gfp, "\n");
+		}
 		return true;
 	}
 
@@ -640,7 +714,14 @@ next_argument:
 	while (*cp1 && !isspace(*cp1))
 		cp1++;
 	if (cp1 != cp) {
-		P("*******************************************************************************");
+		if (html_flag && gfp == stdout) {
+			P("<hr>");
+		} else {
+			P("*******************************************************************************");
+		}
+		if (gfp != stdout) {
+			P("");
+		}
 		/* first, see if the argument matches any command names */
 		flag = false;
 		for (i = 0; i < ARR_CNT(com_list); i++) {
@@ -651,7 +732,7 @@ next_argument:
 			}
 		}
 		if (strncasecmp(cp, "repeat", cp1 - cp) == 0) {
-			P("repeat - Repeat the following command.");
+			P("repeat - Automatically repeat the following command over and over.");
 			P("Usage: repeat command-line");
 			P("Not all commands are repeatable.\n");
 			flag = true;
@@ -663,35 +744,36 @@ next_argument:
 #if	!LIBRARY
 		if (strncasecmp(cp, "startup", cp1 - cp) == 0) {
 			display_startup_message(gfp);
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
 		if (strncasecmp(cp, "options", cp1 - cp) == 0) {
 			usage(gfp);
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
 		if (strncasecmp(cp, "examples", cp1 - cp) == 0) {
 			return read_examples(example_strings);
 		}
 #endif
-		if (strncasecmp(cp, "copyright", cp1 - cp) == 0 || strncasecmp(cp, "license", cp1 - cp) == 0) {
+		if (strncasecmp(cp, "copyrights", cp1 - cp) == 0 || strncasecmp(cp, "licenses", cp1 - cp) == 0) {
 			underline_title(P("Copyright and License for Mathomatic"));
 
 			fprintf(gfp, "%s", license_string);
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
 		if (strncasecmp(cp, "bugs", cp1 - cp) == 0) {
 			P("Please report bugs on the Mathomatic project site on Launchpad.net:");
-			P("https://launchpad.net/mathomatic\n");
+			P("https://launchpad.net/mathomatic");
+			P("Bug tracking is here:");
+			P("https://bugs.launchpad.net/mathomatic\n");
 
 			P("Launchpad features a complete bug management system.");
 			P("Please include the following information (from the version command):\n");
 
 			version_report();
-			cp = skip_space(cp1);
-			goto next_argument;
+
+			P("\nIn Debian, a convenient way to report bugs from the shell command line");
+			P("is \"reportbug mathomatic\".  In Ubuntu, use \"apport-bug mathomatic\".");
+			goto next_space;
 		}
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
 			underline_title(P("Mathomatic Command Usage Syntax"));
@@ -709,8 +791,7 @@ next_argument:
 					}
 				}
 			}
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
 		if (strncasecmp(cp, "geometry", cp1 - cp) == 0) {
 			underline_title(P("Commonly used standard (Euclidean) geometric formulas"));
@@ -734,38 +815,51 @@ next_argument:
 
 			P("Convex polygon of \"n\" sides, sum of all interior angles formula:");
 			P("    sum = (n - 2)*180 degrees     sum = (n - 2)*pi radians");
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
-		if (strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
+		if (cp[0] == '!') {
+			P("A command preceded by an exclamation point (such as \"!ls\") is taken to");
+			P("be a shell command and is passed unchanged to the shell (/bin/sh) when");
+			P("Mathomatic is not in secure mode.  \"!\" by itself invokes the default shell,");
+			P("which is specified in the SHELL environment variable.\n");
+
+			P("\"!\" is also the factorial operator.");
+			goto next_space;
+		}
+		if (is_mathomatic_operator(cp[0]) || strncasecmp(cp, "operators", cp1 - cp) == 0
+		    || strncasecmp(cp, "expressions", cp1 - cp) == 0 || strncasecmp(cp, "equations", cp1 - cp) == 0) {
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
 			P("    ! factorial (gamma function)");
 			P("    ** or ^ power (exponentiation)");
 			P("    * multiply      / divide        % modulus       // integral divide");
-			P("    + add           - subtract");
+			P("    + add           - subtract or negate");
 			P("    = equate (lowest precedence)\n");
 
 			P("Multiple operators of the same precedence level are grouped left to right.");
 			P("Parentheses are used to override operator precedence and group things together.");
 			P("Valid parentheses characters are () and {}.\n");
 
-			P("Variables consist of any combination of letters, digits, and underscores (_).");
-			P("Predefined variables follow:\n");
+			P("Variables consist of any combination of letters, digits, and underscores (_),");
+			P("and \"set special_variable_characters\".  Predefined variables follow:\n");
 
 			P("    sign, sign1, sign2, ... - may only be +1 or -1");
 			P("    integer, integer1, ... - may be any integer value\n");
 
 			P("Absolute value notation \"|x|\" and dual polarity \"+/-x\" are understood.");
-			cp = skip_space(cp1);
-			goto next_argument;
+			P("Type \"help constants\" for help with entering Mathomatic constants.");
+			goto next_space;
 		}
-		if (strncasecmp(cp, "constants", cp1 - cp) == 0) {
-			P("Constants are double precision floating point values with about 14 decimal");
-			P("digits accuracy.  They can be entered in standard, scientific, or hexadecimal");
-			P("notation.  Excepting named constants, constants always start with a decimal");
-			P("digit (0..9) or a period.\n");
+		if (isdigit(cp[0]) || cp[0] == '.'
+		    || strncasecmp(cp, "constants", cp1 - cp) == 0 || strncasecmp(cp, "decimal", cp1 - cp) == 0
+		    || strncasecmp(cp, "floats", cp1 - cp) == 0 || strncasecmp(cp, "doubles", cp1 - cp) == 0
+		    || strncasecmp(cp, "hexadecimal", cp1 - cp) == 0) {
+			P("Constants are displayed as decimal, double precision floating point values");
+			P("with about 14 digits accuracy.  They can be entered in standard, scientific,");
+			P("or hexadecimal notation.  Any constant can be entered in hexadecimal (base 16)");
+			P("by prepending \"0x\" to it.  Excepting named constants, constants always start");
+			P("with a decimal digit (0..9) or a period.\n");
 
 			P("Named constants follow:\n");
 
@@ -774,28 +868,49 @@ next_argument:
 			P("    i or i# - the imaginary unit (square root of -1)");
 			P("The above constants may also be used anywhere variables are required.");
 			P("    inf - floating point infinity constant");
-			P("    nan - invalid floating point result (not enterable)\n");
+			P("    nan - invalid floating point result\n");
 
 			fprintf(gfp, "The largest valid constant is +/-%.7g (slightly less than 2^1024).\n", DBL_MAX);
 			fprintf(gfp, "The smallest valid constant is +/-%.7g or 0.\n", DBL_MIN);
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
-		if (strncasecmp(cp, "table", cp1 - cp) == 0) {
-			fprintf(gfp, "Command\tUsage\tNotes\n");
-			for (i = 0; i < ARR_CNT(com_list); i++) {
-				fprintf(gfp, "%s", com_list[i].name);
-				fprintf(gfp, "\t%s %s", com_list[i].name, com_list[i].usage);
-				if (com_list[i].secondary_name) {
-					fprintf(gfp, "\tAlternate name for this command: %s", com_list[i].secondary_name);
-				}
-				if (com_list[i].extra) {
-					fprintf(gfp, "\t%s", com_list[i].extra);
-				}
-				fprintf(gfp, "\n");
-			}
+		if (strncasecmp(cp, "introduction", cp1 - cp) == 0) {
 			cp = skip_space(cp1);
-			goto next_argument;
+			goto intro;
+		}
+		if (strncasecmp(cp, "colors", cp1 - cp) == 0 || strncasecmp(cp, "colours", cp1 - cp) == 0) {
+			if (color_flag) {
+				fprintf(gfp, "Color mode is currently on, bold mode is currently %s.\n", bold_colors ? "on" : "off");
+				if (html_flag) {
+					fprintf(gfp, "HTML mode is currently on, so color mode is HTML.\n");
+				} else {
+#if	WIN32_CONSOLE_COLORS
+					fprintf(gfp, "Using WIN32 CONSOLE color mode.\n");
+#else
+					fprintf(gfp, "Using ANSI color mode.\n");
+#endif
+				}
+			} else {
+				fprintf(gfp, "Color mode is currently turned off.\n");
+			}
+
+			P("\nColor mode is toggled by the -c option on the shell command line, like this:\n");
+
+			P("    $ mathomatic -c\n");
+
+			P("ANSI color mode is the default, which outputs ANSI terminal escape sequences");
+			P("to make each level of parentheses a different color, improving readability.");
+			P("If ANSI color mode is on, an ANSI compatible terminal emulator is required.");
+			P("If the colors are hard to see, use the -b option instead, which will always");
+			P("turn on bold color mode, increasing the color brightness.\n");
+
+			P("These and other startup command-line options are listed with \"help options\".\n");
+
+			P("The Mathomatic set commands \"set color\" and \"set bold color\" also");
+			P("turn on color and bold color mode for the current session.  \"set no bold\"");
+			P("restores dim colors and \"set no color\" always turns color mode off.");
+			P("When Mathomatic exits, these settings are lost, unless \"set save\" was entered.");
+			goto next_space;
 		}
 		if (is_all(cp)) {
 			underline_title(fprintf(gfp, "Mathomatic Version %s Command Summary\n", VERSION));
@@ -813,12 +928,17 @@ next_argument:
 				}
 			}
 			fprintf(gfp, "End of command list.  Total of %d different commands.\n", ARR_CNT(com_list));
-			cp = skip_space(cp1);
-			goto next_argument;
+			goto next_space;
 		}
 		error(_("Unrecognized help topic or command."));
 		return false;
 	}
 	return true;
+next_space:
+	if (gfp != stdout) {
+		P("");
+	}
+	cp = skip_space(cp1);
+	goto next_argument;
 }
 #endif
