@@ -1,29 +1,32 @@
 /*
  * Mathomatic integration routines and commands.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-static int	integrate_sub();
-static int	laplace_sub();
-static int	inv_laplace_sub();
+static int integrate_sub(token_type *equation, int *np, int loc, int eloc, long v);
+static int laplace_sub(token_type *equation, int *np, int loc, int eloc, long v);
+static int inv_laplace_sub(token_type *equation, int *np, int loc, int eloc, long v);
 
 static int	constant_var_number = 1;	/* makes unique numbers for the constant of integration */
 
@@ -82,7 +85,7 @@ int_dispatch(equation, np, v, func)
 token_type	*equation;	/* pointer to beginning of equation side to integrate */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* integration variable */
-int		(*func)();	/* integration function to call for each term */
+int		(*func)(token_type *equation, int *np, int loc, int eloc, long v);	/* integration function to call for each term */
 {
 	int	i, j;
 
@@ -301,6 +304,7 @@ char	*cp;
 		if (extra_characters(cp))
 			return false;
 	}
+	show_usage = false;
 	if (v == 0) {
 		if (!prompt_var(&v)) {
 			return false;
@@ -310,14 +314,14 @@ char	*cp;
 	if (debug_level >= 0) {
 		list_var(v, 0);
 		if (n_rhs[cur_equation]) {
-			printf(_("Integrating the RHS with respect to (%s)"), var_str);
+			fprintf(gfp, _("Integrating the RHS with respect to (%s)"), var_str);
 		} else {
-			printf(_("Integrating with respect to (%s)"), var_str);
+			fprintf(gfp, _("Integrating with respect to (%s)"), var_str);
 		}
 		if (integrate_order != 1.0) {
-			printf(_(" %.*g times"), precision, integrate_order);
+			fprintf(gfp, _(" %.*g times"), precision, integrate_order);
 		}
-		printf(_(" and simplifying...\n"));
+		fprintf(gfp, _(" and simplifying...\n"));
 	}
 #endif
 	partial_flag = false;
@@ -593,13 +597,15 @@ char	*cp;
 		if (cp == NULL) {
 			return false;
 		}
-	} else {
-		if (!prompt_var(&v)) {
+		if (extra_characters(cp)) {
 			return false;
 		}
 	}
-	if (extra_characters(cp)) {
-		return false;
+	show_usage = false;
+	if (v == 0) {
+		if (!prompt_var(&v)) {
+			return false;
+		}
 	}
 	partial_flag = false;
 	uf_simp(source, nps);
@@ -609,12 +615,12 @@ char	*cp;
 	n1 = *nps;
 	if (inverse_flag) {
 		if (!poly_in_v(dest, n1, v, true) || !int_dispatch(dest, &n1, v, inv_laplace_sub)) {
-			error(_("Inverse Laplace failed."));
+			error(_("Inverse Laplace transformation failed."));
 			return false;
 		}
 	} else {
 		if (!poly_in_v(dest, n1, v, false) || !int_dispatch(dest, &n1, v, laplace_sub)) {
-			error(_("Laplace failed, not a polynomial."));
+			error(_("Laplace transformation failed, not a polynomial."));
 			return false;
 		}
 	}
@@ -639,7 +645,7 @@ int
 nintegrate_cmd(cp)
 char	*cp;
 {
-	long		v = 0;
+	long		v = 0;			/* Mathomatic variable */
 	int		i, j, k, i1, i2;
 	int		len;
 	int		level;
@@ -683,9 +689,12 @@ char	*cp;
 		}
 		if (extra_characters(cp))
 			return false;
-	} else {
-		if (!prompt_var(&v))
+	}
+	show_usage = false;
+	if (v == 0) {
+		if (!prompt_var(&v)) {
 			return false;
+		}
 	}
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
@@ -723,14 +732,14 @@ char	*cp;
 	}
 #if	!SILENT
 	if (n_rhs[cur_equation]) {
-		printf(_("Approximating the definite integral of the RHS\n"));
+		fprintf(gfp, _("Approximating the definite integral of the RHS\n"));
 	} else {
-		printf(_("Approximating the definite integral\n"));
+		fprintf(gfp, _("Approximating the definite integral numerically\n"));
 	}
 	if (trap_flag) {
-		printf(_("using the trapezoid method (%d partitions)...\n"), iterations);
+		fprintf(gfp, _("using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
-		printf(_("using Simpson's rule (%d partitions)...\n"), iterations);
+		fprintf(gfp, _("using Simpson's rule (%d partitions)...\n"), iterations);
 	}
 #endif
 	subst_constants(source, nps);
@@ -841,7 +850,9 @@ char	*cp;
 		elim_loop(dest, &n1);
 		ufactor(dest, &n1);
 		simp_divide(dest, &n1);
+		factor_imaginary(dest, &n1);
 		approximate_roots = false;
+		side_debug(1, dest, n1);
 
 		if (exp_contains_infinity(dest, n1)) {
 			error(_("Integration failed because result contains infinity or NaN (a singularity)."));
@@ -858,7 +869,7 @@ char	*cp;
 			break;
 		default:
 			if ((n1 / 8) >= first_size) {
-				error(_("Integration failed."));
+				error(_("Result growing, integration failed."));
 				return false;
 			}
 			break;
@@ -899,7 +910,7 @@ char	*cp;
 	approximate_roots = false;
 
 #if	!SILENT
-	printf(_("Numerical integration successful:\n"));
+	fprintf(gfp, _("Numerical integration successful:\n"));
 #endif
 	*np = n1;
 	if (n_rhs[cur_equation]) {
@@ -913,6 +924,5 @@ char	*cp;
 			parse_var(&lhs[i][0].token.variable, var_str);
 		}
 	}
-	cur_equation = i;
-	return return_result(cur_equation);
+	return return_result(i);
 }
