@@ -2,42 +2,69 @@
  * Mathomatic expression and equation display routines.
  * Color mode routines, too.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-#define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
-#define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
+#if	WIN32_CONSOLE_COLORS
+/* The WIN32_CONSOLE_COLORS code was contributed by Doug Snead for the MinGW C compiler. */
+#include <windows.h>
+#include <wincon.h> 
 
-static int	flist_sub();
-static int	flist_recurse();
+#define  FOREGROUND_BLACK	0
+#define  FOREGROUND_YELLOW      (FOREGROUND_RED|FOREGROUND_GREEN)  
+#define  FOREGROUND_MAGENTA     (FOREGROUND_BLUE|FOREGROUND_RED)   
+#define  FOREGROUND_CYAN        (FOREGROUND_BLUE|FOREGROUND_GREEN) 
+#define  FOREGROUND_WHITE       (FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE)
 
+/* MS-Windows color array for coloring mathematical expressions, warnings, and errors. */
+static short windows_carray[] = {
+    FOREGROUND_GREEN, 
+    FOREGROUND_YELLOW,	/* warning text color */
+    FOREGROUND_RED, 	/* error text color */
+    FOREGROUND_MAGENTA,
+    FOREGROUND_BLUE, 
+    FOREGROUND_CYAN,
+}; 
+
+extern HANDLE hOut;
+#else
 /* ANSI terminal color code array for 8 color ANSI; we don't use black or white */
 /* because the background may be the same color, so there are only 6 colors here. */
 static int	carray[] = {
 	32,	/* must be green (default color) */
 	33,	/* must be yellow (for warnings) */
 	31,	/* must be red (for errors) */
+	34,	/* must be blue (for prompts) */
 	35,	/* magenta */
-	34,	/* blue */
-	36	/* cyan */
+	36,	/* cyan */
 };
+#endif
+
+#define	EQUATE_STRING	" = "	/* string displayed between the LHS and RHS of equations */
+#define MODULUS_STRING	" % "	/* string displayed for the modulus operator */
+
+static int flist_sub(token_type *p1, int n, int out_flag, char *string, int sbuffer_size, int pos, int *highp, int *lowp);
+static int flist_recurse(token_type *p1, int n, int out_flag, char *string, int sbuffer_size, int line, int pos, int cur_level, int *highp, int *lowp);
 
 /* Bright HTML color array. */
 /* Used when HTML output and "set color" and "set bold" options are enabled. */
@@ -46,10 +73,10 @@ static char	*bright_html_carray[] = {
 	"#00FF00",	/* must be bright green (default color) */
 	"#FFFF00",	/* must be bright yellow (for warnings) */
 	"#FF0000",	/* must be bright red (for errors) */
+	"#0000FF",	/* must be bright blue (for prompts) */
 	"#FF9000",
 	"#FF00FF",
 	"#00FFFF",
-	"#0000FF"
 };
 
 /* Dim HTML color array for color HTML output. */
@@ -59,16 +86,12 @@ static char	*html_carray[] = {
 	"green",
 	"olive",
 	"red",
+	"navy",
 	"maroon",
 	"purple",
 	"teal",
-	"navy"
 };
 
-/* global variables for flist_equation() below */
-static int	cur_line;	/* current line */
-static int	cur_pos;	/* current position in the current line on the screen */
-
 /*
  * Reset terminal attributes function.
  * Turn color off if color mode is on.
@@ -82,7 +105,11 @@ reset_attr()
 		if (html_flag) {
 			printf("</font>");
 		} else {
-			printf("\033[0m");
+#if	WIN32_CONSOLE_COLORS
+                        SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+#else
+                        printf("\033[0m");
+#endif
 		}
 		fflush(stdout);	/* flush standard output */
 	}
@@ -113,7 +140,14 @@ int	color;
 				printf("<font color=\"%s\">", html_carray[color % ARR_CNT(html_carray)]);
 			}
 		} else {
-			printf("\033[%d;%dm", bold_colors, carray[color % ARR_CNT(carray)]);
+#if	WIN32_CONSOLE_COLORS
+                        short attr = windows_carray[color % ARR_CNT(windows_carray)];
+                        if (bold_colors)
+                                 attr |= FOREGROUND_INTENSITY;
+                        SetConsoleTextAttribute(hOut, attr);
+#else
+                        printf("\033[%d;%dm", bold_colors, carray[color % ARR_CNT(carray)]);
+#endif
 		}
 		cur_color = color;
 	}
@@ -121,12 +155,24 @@ int	color;
 
 /*
  * Set normal text color for subsequent output.
- * Currently, the normal color (0) is green.
  */
 void
 default_color()
 {
-	set_color(0);
+	if (gfp == stdout) {
+		if (color_flag && cur_color >= 0) {
+			if (html_flag) {
+				printf("</font>");
+			} else {
+#if	WIN32_CONSOLE_COLORS
+                                SetConsoleTextAttribute(hOut, FOREGROUND_WHITE);
+#else
+                                printf("\033[0m");
+#endif
+			}
+		}
+		cur_color = -1;
+	}
 	fflush(stdout);
 }
 
@@ -161,7 +207,7 @@ int
 list1_sub(n, export_flag)
 int	n;		/* equation space number */
 int	export_flag;	/* non-zero for exportable format (readable by other math programs) */
-			/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
+			/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
 	int	len = 0;
 
@@ -178,7 +224,7 @@ int	export_flag;	/* non-zero for exportable format (readable by other math progr
 	if (export_flag == 1) {
 		len += fprintf(gfp, ";");
 	}
-#if	CYGWIN
+#if	CYGWIN || MINGW
 	fprintf(gfp, "\r\n");	/* might be redirecting to a Microsoft text file */
 #else
 	fprintf(gfp, "\n");
@@ -261,7 +307,7 @@ long	v;	/* Mathomatic variable */
  *
  * If (lang_code == 0), use standard Mathomatic format.
  * If (lang_code > 0), make variable compatible with the language defined in the enumeration language_list defined in am.h
- * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for Gnuplot, -4 for hexadecimal.
+ * If (lang_code < 0), create an exportable variable name: -1 for Maxima, -2 for other, -3 for gnuplot, -4 for hexadecimal.
  */
 int
 list_var(v, lang_code)
@@ -284,16 +330,14 @@ int	lang_code;	/* language code */
 		case -3:
 			cp = "{0,1}";
 			break;
+		case 0:
+		case -4:
 		case -2:
 			cp = "i";
 			break;
 		case -1:
 			cp = "%i";
 			break;
-		case 0:
-		case -4:
-			cp = "i#";
-			break;
 		case PYTHON:
 			cp = "1j";
 			break;
@@ -310,10 +354,6 @@ int	lang_code;	/* language code */
 		case -1:
 			cp = "%e";
 			break;
-		case 0:
-		case -4:
-			cp = "e#";
-			break;
 		case C:
 			cp ="M_E";
 			break;
@@ -333,10 +373,6 @@ int	lang_code;	/* language code */
 		case -1:
 			cp = "%pi";
 			break;
-		case 0:
-		case -4:
-			cp = "pi#";
-			break;
 		case C:
 			cp = "M_PI";
 			break;
@@ -371,8 +407,8 @@ int	lang_code;	/* language code */
 	}
 /* Make valid C type variable if exporting or generating code: */
 	if (from_memory && lang_code != 0 && lang_code != -4) {
-		for (j = 0; var_str[j]; j++) {
-			if (var_str[j] != '_' && !isalnum(var_str[j])) {
+		for (j = 0; var_str[j] && var_str[j] != '('; j++) {
+			if (strchr("_[]", var_str[j]) == NULL && !isalnum(var_str[j])) {
 				var_str[j] = '_';
 			}
 		}
@@ -391,16 +427,16 @@ list_proc(p1, n, export_flag)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
+				/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
 	return list_string_sub(p1, n, true, NULL, export_flag);
 }
 
 /*
- * Store the specified equation in a text string.
+ * Store the expression from the specified equation space in a text string in single-line format.
  * String should be freed with free() when done.
  *
- * Return string or NULL if error.
+ * Returns text string, or NULL if error.
  */
 char *
 list_equation(n, export_flag)
@@ -439,7 +475,7 @@ int	export_flag;	/* flag for exportable format (usually false) */
  * Store an expression in a text string.
  * String should be freed with free() when done.
  *
- * Return string or NULL if error.
+ * Return string, or NULL if error.
  */
 char *
 list_expression(p1, n, export_flag)
@@ -482,6 +518,9 @@ int		export_flag;
 	return list_string_sub(p1, n, false, string, export_flag);
 }
 
+#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
+#define	APPEND2(str)	{ if (string) { if ((sbuffer_size - current_len) > 0) my_strlcpy(&string[current_len], str, sbuffer_size - current_len); } else { fprintf(gfp, "%s", str); } current_len += strlen(str); }
+
 int
 list_string_sub(p1, n, outflag, string, export_flag)
 token_type	*p1;		/* expression pointer */
@@ -489,10 +528,8 @@ int		n;		/* length of expression */
 int		outflag;	/* if true, output to gfp */
 char		*string;	/* buffer to save output to or NULL pointer */
 int		export_flag;	/* flag for exportable format (usually false) */
-				/* 1 for Maxima, 2 for other, 3 for Gnuplot, 4 for hexadecimal */
+				/* 1 for Maxima, 2 for other, 3 for gnuplot, 4 for hexadecimal */
 {
-#define	APPEND(str)	{ if (string) { strcpy(&string[len], str); } if (outflag) { fprintf(gfp, "%s", str); } len += strlen(str); }
-
 	int	i, j, k, i1;
 	int	min1;
 	int	cur_level;
@@ -503,6 +540,8 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	int	cflag, power_flag;
 
 	cflag = (outflag && (export_flag == 0 || export_flag == 4));
+	if (cflag)
+		set_color(0);
 	if (string)
 		string[0] = '\0';
 	if (high_prec)
@@ -512,7 +551,6 @@ int		export_flag;	/* flag for exportable format (usually false) */
 	cur_level = min1 = min_level(p1, n);
 	for (i = 0; i < n; i++) {
 		power_flag = false;
-#if	1
 		if (export_flag == 0 && !high_prec) {
 			for (j = i - 1; j <= (i + 1); j++) {
 				if ((j - 1) >= 0 && (j + 1) < n
@@ -526,7 +564,6 @@ int		export_flag;	/* flag for exportable format (usually false) */
 				}
 			}
 		}
-#endif
 		j = cur_level - p1[i].level;
 		if (power_flag)
 			k = abs(j) - 1;
@@ -623,6 +660,8 @@ int		export_flag;	/* flag for exportable format (usually false) */
 		if (cflag)
 			set_color(j);
 	}
+	if (cflag)
+		default_color();
 	return len;
 }
 
@@ -662,7 +701,7 @@ int		n;		/* length of expression */
  * Return length of output (number of characters).
  */
 int
-list_c_equation(en, language, int_flag)
+list_code_equation(en, language, int_flag)
 int			en;		/* equation space number */
 enum language_list	language;
 int			int_flag;	/* integer arithmetic flag */
@@ -671,10 +710,10 @@ int			int_flag;	/* integer arithmetic flag */
 
 	if (n_lhs[en] <= 0)
 		return 0;
-	len += list_code(lhs[en], &n_lhs[en], language, int_flag);
+	len += list_code(lhs[en], &n_lhs[en], true, NULL, language, int_flag);
 	if (n_rhs[en]) {
-		len += fprintf(gfp, " = ");
-		len += list_code(rhs[en], &n_rhs[en], language, int_flag);
+		len += fprintf(gfp, EQUATE_STRING);
+		len += list_code(rhs[en], &n_rhs[en], true, NULL, language, int_flag);
 	}
 	switch (language) {
 	case C:
@@ -688,6 +727,51 @@ int			int_flag;	/* integer arithmetic flag */
 	return len;
 }
 
+/*
+ * Convert the specified equation space to a string of C, Java, or Python code.
+ * String should be freed with free() when done.
+ *
+ * Return string, or NULL if error.
+ */
+char *
+string_code_equation(n, language, int_flag)
+int			n;		/* equation space number */
+enum language_list	language;
+int			int_flag;	/* integer arithmetic flag */
+{
+	int	len;
+	char	*cp;
+
+	if (n_lhs[n] <= 0) {
+		return NULL;
+	}
+	len = list_code(lhs[n], &n_lhs[n], false, NULL, language, int_flag);
+	if (n_rhs[n]) {
+		len += strlen(EQUATE_STRING);
+		len += list_code(rhs[n], &n_rhs[n], false, NULL, language, int_flag);
+	}
+	len += 2;	/* for possible semicolon and terminating null character */
+	cp = (char *) malloc(len);
+	if (cp == NULL) {
+		error(_("Out of memory (can't malloc(3))."));
+		return NULL;
+	}
+	list_code(lhs[n], &n_lhs[n], false, cp, language, int_flag);
+	if (n_rhs[n]) {
+		strcat(cp, EQUATE_STRING);
+		list_code(rhs[n], &n_rhs[n], false, &cp[strlen(cp)], language, int_flag);
+	}
+	switch (language) {
+	case C:
+	case JAVA:
+		strcat(cp, ";");
+		break;
+	default:
+		break;
+	}
+	return cp;
+}
+
 /*
  * Output C, Java, or Python code for an expression.
  * Expression might be modified by this function, though it remains equivalent.
@@ -695,9 +779,11 @@ int			int_flag;	/* integer arithmetic flag */
  * Return length of output (number of characters).
  */
 int
-list_code(equation, np, language, int_flag)
+list_code(equation, np, outflag, string, language, int_flag)
 token_type		*equation;	/* equation side pointer */
 int			*np;		/* pointer to length of equation side */
+int			outflag;	/* if true, output to gfp */
+char			*string;	/* buffer to save output to or NULL pointer */
 enum language_list	language;	/* see enumeration language_list in am.h */
 int			int_flag;	/* integer arithmetic flag, should work with any language */
 {
@@ -705,9 +791,11 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 	int	min1;
 	int	cur_level;
 	char	*cp;
-	char	buf[500];
+	char	buf[500], buf2[500];
 	int	len = 0;
 
+	if (string)
+		string[0] = '\0';
 	min1 = min_level(equation, *np);
 	if (*np > 1)
 		min1--;
@@ -718,7 +806,7 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 		for (i1 = 1; i1 <= k; i1++) {
 			if (j > 0) {
 				cur_level--;
-				len += fprintf(gfp, ")");
+				APPEND(")");
 			} else {
 				cur_level++;
 				for (i2 = i + 1; i2 < *np && equation[i2].level >= cur_level; i2 += 2) {
@@ -735,10 +823,10 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 								if (!int_flag) {
 									switch (language) {
 									case C:
-										len += fprintf(gfp, "pow");
+										APPEND("pow");
 							 			break;
 									case JAVA:
-										len += fprintf(gfp, "Math.pow");
+										APPEND("Math.pow");
 										break;
 									default:
 										break;
@@ -747,13 +835,13 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 							}
 							break;
 						case FACTORIAL:
-							len += fprintf(gfp, "fact");
+							APPEND("fact");
 							break;
 						}
 						break;
 					}
 				}
-				len += fprintf(gfp, "(");
+				APPEND("(");
 			}
 		}
 		switch (equation[i].kind) {
@@ -769,17 +857,18 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 			}
 /* Here we will need to parenthesize negative numbers to make -2**x work the same with Python: */
 			if (equation[i].token.constant < 0) {
-				len += fprintf(gfp, "(%s)", buf);
+				snprintf(buf2, sizeof(buf2), "(%s)", buf);
+				APPEND(buf2);
 			} else {
-				len += fprintf(gfp, "%s", buf);
+				APPEND(buf);
 			}
 			break;
 		case VARIABLE:
 		  	if (int_flag && (language == C || language == JAVA) && equation[i].token.variable == IMAGINARY) {
-				len += fprintf(gfp, "1i");
+				APPEND("1i");
 			} else {
 				list_var(equation[i].token.variable, language);
-				len += fprintf(gfp, "%s", var_str);
+				APPEND(var_str);
 			}
 			break;
 		case OPERATOR:
@@ -794,8 +883,12 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 			case TIMES:
 				cp = "*";
 				break;
-			case DIVIDE:
 			case IDIVIDE:
+				if (language == PYTHON) {
+					cp = "//";
+					break;
+				}
+			case DIVIDE:
 				cp = "/";
 				break;
 			case MODULUS:
@@ -813,16 +906,99 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 				i++;
 				break;
 			}
-			len += fprintf(gfp, "%s", cp);
+			APPEND(cp);
 			break;
 		}
 	}
 	for (j = cur_level - min1; j > 0; j--) {
-		len += fprintf(gfp, ")");
+		APPEND(")");
 	}
 	return len;
 }
 
+/* global variables for the flist functions below */
+static int	cur_line;	/* current line */
+static int	cur_pos;	/* current position in the current line on the screen */
+
+#if	LIBRARY
+/*
+ * Return a multi-line C string containing the specified equation space in 2D multi-line fraction format.
+ * The string should be freed after use with free(3).
+ *
+ * Color mode is not used.
+ *
+ * The equation sides must first be basically simplified and prepared by fractions_and_group(),
+ * for proper formatting.
+ *
+ * Return NULL on failure.
+ * Result will be limited to TEXT_COLUMNS columns, TEXT_ROWS lines.
+ * Exceeding TEXT_COLUMNS will result in truncation,
+ * exceeding TEXT_ROWS will result in failure and NULL return.
+ */
+char *
+flist_equation_string(n)
+int	n;	/* equation space number */
+{
+	int	i;
+	int	len, cur_len, buf_len;
+	int	pos;
+	int	high = 0, low = 0;
+	int	max_line = 0, min_line = 0;
+	int	screen_line;
+	char	*cp;
+
+	if (n_lhs[n] <= 0)
+		return NULL;
+	for (i = 0; i < TEXT_ROWS; i++) {
+		vscreen[i][0] = '\0';
+	}
+	cur_line = 0;
+	cur_pos = 0;
+	len += flist_sub(lhs[n], n_lhs[n], false, NULL, TEXT_COLUMNS, 0, &max_line, &min_line);
+	if (n_rhs[n]) {
+		len += strlen(EQUATE_STRING);
+		len += flist_sub(rhs[n], n_rhs[n], false, NULL, TEXT_COLUMNS, 0, &high, &low);
+		if (high > max_line)
+			max_line = high;
+		if (low < min_line)
+			min_line = low;
+	}
+	if ((max_line - min_line) >= TEXT_ROWS)
+		return NULL;
+	for (cur_line = max_line, screen_line = 0; cur_line >= min_line; cur_line--, screen_line++) {
+		pos = cur_pos = 0;
+		pos += flist_sub(lhs[n], n_lhs[n], true, vscreen[screen_line], TEXT_COLUMNS, pos, &high, &low);
+		if (n_rhs[n]) {
+			if (cur_line == 0) {
+				cur_pos += strlen(EQUATE_STRING);
+				cur_len = strlen(vscreen[screen_line]);
+				if ((TEXT_COLUMNS - cur_len) > 0)
+					my_strlcpy(&vscreen[screen_line][cur_len], EQUATE_STRING, TEXT_COLUMNS - cur_len);
+			}
+			pos += strlen(EQUATE_STRING);
+			pos += flist_sub(rhs[n], n_rhs[n], true, vscreen[screen_line], TEXT_COLUMNS, pos, &high, &low);
+		}
+	}
+	if (screen_line <= 0)
+		return NULL;
+	for (i = 0, buf_len = 1; i < screen_line; i++) {
+		buf_len += strlen(vscreen[i]);
+		buf_len++;	/* For newlines */
+	}
+	cp = (char *) malloc(buf_len);
+	if (cp == NULL) {
+		error(_("Out of memory (can't malloc(3))."));
+		return NULL;
+	}
+	cp[0] = '\0';
+	for (i = 0; i < screen_line; i++) {
+		strcat(cp, vscreen[i]);
+		strcat(cp, "\n");
+	}
+	return(cp);
+}
+#endif
+
 /*
  * Display an equation space in 2D multi-line fraction format.
  * Use color if color mode is set.
@@ -851,11 +1027,11 @@ int	n;	/* equation space number */
 	cur_line = 0;
 	cur_pos = 0;
 	sind = n_rhs[n];
-	len += flist_sub(lhs[n], n_lhs[n], false, 0, &max_line, &min_line);
+	len += flist_sub(lhs[n], n_lhs[n], false, NULL, screen_columns, 0, &max_line, &min_line);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
 make_smaller:
-		len2 = flist_sub(rhs[n], sind, false, 0, &high, &low);
+		len2 = flist_sub(rhs[n], sind, false, NULL, screen_columns, 0, &high, &low);
 		if (screen_columns && gfp == stdout && (len + len2) >= screen_columns && sind > 0) {
 			for (sind--; sind > 0; sind--) {
 				if (rhs[n][sind].level == 1 && rhs[n][sind].kind == OPERATOR) {
@@ -873,7 +1049,7 @@ make_smaller:
 			max_line = high;
 		if (low < min_line)
 			min_line = low;
-		len3 = flist_sub(&rhs[n][sind], n_rhs[n] - sind, false, 0, &max2_line, &min2_line);
+		len3 = flist_sub(&rhs[n][sind], n_rhs[n] - sind, false, NULL, screen_columns, 0, &max2_line, &min2_line);
 	} else {
 		len2 = 0;
 		len3 = 0;
@@ -882,7 +1058,7 @@ make_smaller:
 	if (screen_columns && gfp == stdout && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
 		width = list1_sub(n, false);
-#if	CYGWIN
+#if	CYGWIN || MINGW
 		fprintf(gfp, "\r\n");	/* Be consistent with list1_sub() output. */
 #else
 		fprintf(gfp, "\n");
@@ -896,13 +1072,13 @@ make_smaller:
 			cur_pos += fprintf(gfp, "%s", buf);
 		}
 		pos += strlen(buf);
-		pos += flist_sub(lhs[n], n_lhs[n], true, pos, &high, &low);
+		pos += flist_sub(lhs[n], n_lhs[n], true, NULL, screen_columns, pos, &high, &low);
 		if (n_rhs[n]) {
 			if (cur_line == 0) {
 				cur_pos += fprintf(gfp, "%s", EQUATE_STRING);
 			}
 			pos += strlen(EQUATE_STRING);
-			pos += flist_sub(rhs[n], sind, true, pos, &high, &low);
+			pos += flist_sub(rhs[n], sind, true, NULL, screen_columns, pos, &high, &low);
 		}
 		fprintf(gfp, "\n");
 	}
@@ -911,7 +1087,7 @@ make_smaller:
 		fprintf(gfp, "\n");
 		for (cur_line = max2_line; cur_line >= min2_line; cur_line--) {
 			cur_pos = 0;
-			flist_sub(&rhs[n][sind], n_rhs[n] - sind, true, 0, &high, &low);
+			flist_sub(&rhs[n][sind], n_rhs[n] - sind, true, NULL, screen_columns, 0, &high, &low);
 			fprintf(gfp, "\n");
 		}
 	}
@@ -925,32 +1101,36 @@ make_smaller:
  * 0 is the middle line, lines above are positive, lines below are negative.
  * Use color if available.
  *
- * This function is for internal use only, and not to be called, except by flist_equation().
+ * The following functions are for internal use only, and not to be called, except by flist_equation().
  *
  * Return the width of the expression (that is, the required number of screen columns).
  */
 static int
-flist_sub(p1, n, out_flag, pos, highp, lowp)
+flist_sub(p1, n, out_flag, string, sbuffer_size, pos, highp, lowp)
 token_type	*p1;		/* expression pointer */
 int		n;		/* length of expression */
-int		out_flag;
+int		out_flag;	/* if true, output to gfp or string */
+char		*string;	/* if not NULL, put output to here instead of gfp */
+int		sbuffer_size;	/* string buffer size */
 int		pos;
 int		*highp, *lowp;
 {
-	int	i;
+	int	rv;
 
-	i = flist_recurse(p1, n, out_flag, 0, pos, 1, highp, lowp);
-	if (out_flag) {
+	rv = flist_recurse(p1, n, out_flag, string, sbuffer_size, 0, pos, 1, highp, lowp);
+	if (out_flag && (string == NULL)) {
 		default_color();
 	}
-	return i;
+	return rv;
 }
 
 static int
-flist_recurse(p1, n, out_flag, line, pos, cur_level, highp, lowp)
+flist_recurse(p1, n, out_flag, string, sbuffer_size, line, pos, cur_level, highp, lowp)
 token_type	*p1;
 int		n;
-int		out_flag;
+int		out_flag;	/* if true, output to gfp or string */
+char		*string;	/* if not NULL, put output to here instead of gfp */
+int		sbuffer_size;	/* string buffer size */
 int		line;
 int		pos;
 int		cur_level;
@@ -964,12 +1144,15 @@ int		*highp, *lowp;
 	int	len_div;
 	int	level;
 	int	start_level;
-	int	oflag, html_out, power_flag;
+	int	oflag, cflag, html_out, power_flag;
 	int	len = 0, len1, len2;
+	int	current_len = 0;
 	int	high, low;
 	char	buf[500];
 	char	*cp;
 
+	if (string)
+		current_len = strlen(string);
 	start_level = cur_level;
 	*highp = line;
 	*lowp = line;
@@ -977,11 +1160,13 @@ int		*highp, *lowp;
 		return 0;
 	}
 	oflag = (out_flag && line == cur_line);
+	cflag = (oflag && string == NULL);
 	html_out = (html_flag && gfp == stdout);
 	if (oflag) {
-		set_color(cur_level-1);
+		if (cflag)
+			set_color(cur_level-1);
 		for (; cur_pos < pos; cur_pos++) {
-			fprintf(gfp, " ");
+			APPEND2(" ");
 		}
 	}
 	ii = 0;
@@ -1013,10 +1198,14 @@ check_again:
 	for (i = ii; i < n; i++) {
 		power_flag = false;
 		if (i == stop_at) {
+#if	DEBUG
+			if (div_loc < 0) {
+				error_bug("Bug in flist_recurse().");
+			}
+#endif
 			j = cur_level - p1[div_loc].level;
 			k = abs(j) - 1;
 		} else {
-#if	1
 			for (j = i - 1; j <= (i + 1); j++) {
 				if ((j - 1) >= ii && (j + 1) < n
 				    && p1[j].kind == OPERATOR && (p1[j].token.operatr == POWER || p1[j].token.operatr == FACTORIAL)
@@ -1028,7 +1217,6 @@ check_again:
 					break;
 				}
 			}
-#endif
 			j = cur_level - p1[i].level;
 			if (power_flag)
 				k = abs(j) - 1;
@@ -1040,27 +1228,29 @@ check_again:
 				cur_level--;
 				len++;
 				if (oflag) {
-					fprintf(gfp, ")");
-					set_color(cur_level-1);
+					APPEND2(")");
+					if (cflag)
+						set_color(cur_level-1);
 				}
 			} else {
 				cur_level++;
 				len++;
 				if (oflag) {
-					set_color(cur_level-1);
-					fprintf(gfp, "(");
+					if (cflag)
+						set_color(cur_level-1);
+					APPEND2("(");
 				}
 			}
 		}
 		if (i == stop_at) {
 			level = p1[div_loc].level;
-			len1 = flist_recurse(&p1[stop_at], div_loc - stop_at, false, line + 1, pos + len, level, &high, &low);
+			len1 = flist_recurse(&p1[stop_at], div_loc - stop_at, false, string, sbuffer_size, line + 1, pos + len, level, &high, &low);
 			l1 = (2 * (line + 1)) - low;
 			for (j = div_loc + 2; j < n; j += 2) {
 				if (p1[j].level <= level)
 					break;
 			}
-			len2 = flist_recurse(&p1[div_loc+1], j - (div_loc + 1), false, line - 1, pos + len, level, &high, &low);
+			len2 = flist_recurse(&p1[div_loc+1], j - (div_loc + 1), false, string, sbuffer_size, line - 1, pos + len, level, &high, &low);
 			l2 = (2 * (line - 1)) - high;
 			ii = j;
 			len_div = max(len1, len2);
@@ -1068,24 +1258,30 @@ check_again:
 			if (len1 < len_div) {
 				j = (len_div - len1) / 2;
 			}
-			flist_recurse(&p1[stop_at], div_loc - stop_at, out_flag, l1, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[stop_at], div_loc - stop_at, out_flag, string, sbuffer_size, l1, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
 				*lowp = low;
 			if (oflag) {
 				/* display fraction bar */
-				set_color(level-1);
+				if (cflag)
+					set_color(level-1);
 				for (j = 0; j < len_div; j++) {
-					fprintf(gfp, "-");
+					if (html_out) {
+						APPEND2("&ndash;");
+					} else {
+						APPEND2("-");
+					}
 				}
-				set_color(cur_level-1);
+				if (cflag)
+					set_color(cur_level-1);
 			}
 			j = 0;
 			if (len2 < len_div) {
 				j = (len_div - len2) / 2;
 			}
-			flist_recurse(&p1[div_loc+1], ii - (div_loc + 1), out_flag, l2, pos + len + j, level, &high, &low);
+			flist_recurse(&p1[div_loc+1], ii - (div_loc + 1), out_flag, string, sbuffer_size, l2, pos + len + j, level, &high, &low);
 			if (high > *highp)
 				*highp = high;
 			if (low < *lowp)
@@ -1098,7 +1294,15 @@ check_again:
 			if (p1[i].token.constant == 0.0) {
 				p1[i].token.constant = 0.0; /* fix -0 */
 			}
-			if (p1[i].token.constant == -1.0 && (i == 0 || p1[i-1].level < p1[i].level)
+			if (html_out && isinf(p1[i].token.constant)) {
+				if (p1[i].token.constant < 0) {
+					snprintf(buf, sizeof(buf), "(-&infin;)");
+					len += 4;
+				} else {
+					snprintf(buf, sizeof(buf), "&infin;");
+					len += 1;
+				}
+			} else if (p1[i].token.constant == -1.0 && (i == 0 || p1[i-1].level < p1[i].level)
 			    && (i + 1) < n && p1[i].level == p1[i+1].level && p1[i+1].token.operatr == TIMES) {
 				i++;
 				len += snprintf(buf, sizeof(buf), "-");
@@ -1117,12 +1321,26 @@ check_again:
 				}
 			}
 			if (oflag)
-				fprintf(gfp, "%s", buf);
+				APPEND2(buf);
 			break;
 		case VARIABLE:
-			len += list_var(p1[i].token.variable, 0);
-			if (oflag)
-				fprintf(gfp, "%s", var_str);
+			if (html_out && p1[i].token.variable == V_PI) {
+				len += 1;
+				if (oflag)
+					APPEND2("&pi;");
+			} else if (html_out && p1[i].token.variable == V_E) {
+				len += 1;
+				if (oflag)
+					APPEND2("&ecirc;");
+			} else if (html_out && p1[i].token.variable == IMAGINARY) {
+				len += 1;
+				if (oflag)
+					APPEND2("&icirc;");
+			} else {
+				len += list_var(p1[i].token.variable, 0);
+				if (oflag)
+					APPEND2(var_str);
+			}
 			break;
 		case OPERATOR:
 /* "len" must be incremented here by the number of columns the display of the operator takes */
@@ -1161,22 +1379,13 @@ check_again:
 				break;
 			case POWER:
 #if	0
-				if (html_out && power_flag
-				    && p1[i-1].level == p1[i].level
+				if (html_out
 				    && p1[i+1].level == p1[i].level
 				    && p1[i+1].kind == CONSTANT) {
-					if (p1[i+1].token.constant == 2.0) {
-						cp = "&sup2;";
-						len++;
-						i++;
-						break;
-					}
-					if (p1[i+1].token.constant == 3.0) {
-						cp = "&sup3;";
-						len++;
-						i++;
-						break;
-					}
+					len += (snprintf(buf, sizeof(buf), "<sup>%.*g</sup>", precision, p1[i+1].token.constant) - 11);
+					cp = buf;
+					i++;
+					break;
 				}
 #endif
 				if (power_starstar) {
@@ -1198,7 +1407,7 @@ check_again:
 				break;
 			}
 			if (oflag)
-				fprintf(gfp, "%s", cp);
+				APPEND2(cp);
 			break;
 		}
 	}
@@ -1207,8 +1416,8 @@ check_again:
 		len++;
 		j--;
 		if (oflag) {
-			fprintf(gfp, ")");
-			if (j > 0)
+			APPEND2(")");
+			if (j > 0 && cflag)
 				set_color(cur_level-1);
 		}
 	}
