@@ -1,22 +1,25 @@
 /*
  * This file contains main() and the startup code for Mathomatic,
- * which is a small computer algebra system written entirely in C.
+ * which is a computer algebra system written in the C programming language.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -59,7 +62,16 @@
 #if	!LIBRARY	/* This comments out this whole file if compiling as the symbolic math library. */
 
 #include "includes.h"
+#if	!NO_GETOPT_H
 #include <getopt.h>
+#endif
+
+#if	WIN32_CONSOLE_COLORS
+#include <windows.h>
+#include <wincon.h>
+
+HANDLE	hOut;
+#endif
 
 void
 display_startup_message(fp)
@@ -70,10 +82,12 @@ FILE	*fp;
 #else
 	if (security_level >= 2)
 		fprintf(fp, _("Secure "));
+	else if (security_level == -1)
+		fprintf(fp, "m4 ");
 #endif
-	fprintf(fp, "Mathomatic version %s (www.mathomatic.org)\n", VERSION);
-	fprintf(fp, "Copyright (C) 1987-2010 George Gesslein II.\n");
-	fprintf(fp, _("%d equation spaces available, %ld kilobytes per equation space.\n"),
+	fprintf(fp, "Mathomatic version %s\n", VERSION);
+	fprintf(fp, "Copyright (C) 1987-2011 George Gesslein II.\n");
+	fprintf(fp, _("%d equation spaces available in memory,\n%ld kilobytes per equation space.\n"),
 	    N_EQUATIONS, (long) n_tokens * sizeof(token_type) * 2L / 1000L);
 }
 
@@ -84,7 +98,7 @@ void
 usage(fp)
 FILE	*fp;
 {
-	fprintf(fp, _("Mathomatic computer algebra system\n"));
+	fprintf(fp, _("Mathomatic computer algebra system, version %s\n"), VERSION);
 	fprintf(fp, _("Usage: %s [ options ] [ input_files or input ]\n\n"), prog_name);
 	fprintf(fp, _("Options:\n"));
 	fprintf(fp, _("  -b            Enable bold color mode.\n"));
@@ -100,7 +114,7 @@ FILE	*fp;
 	fprintf(fp, _("  -v            Display version information, then exit.\n"));
 	fprintf(fp, _("  -w            Wide output mode, sets unlimited width.\n"));
 	fprintf(fp, _("  -x            Enable HTML/XHTML output mode.\n"));
-	fprintf(fp, _("\nPlease refer to the Mathomatic man page for more information.\n"));
+	fprintf(fp, _("\nPlease refer to the Mathomatic man page for details.\n"));
 	fprintf(fp, _("The man page is viewed by typing \"man mathomatic\" in shell.\n"));
 }
 
@@ -109,24 +123,30 @@ main(argc, argv)
 int	argc;
 char	**argv;
 {
+#if	NO_GETOPT_H	/* if no getopt.h is available */
 	extern char	*optarg;	/* set by getopt(3) */
 	extern int	optind;
+#endif
 
 	int		i;
 	char		*cp = NULL;
 	double		numerator, denominator;
-	double		new_size;
+	double		new_size = 0;
 	int		coption = false, boption = false, wide_flag = false;
 	int		exit_value = 0;
 
+#if	WIN32_CONSOLE_COLORS
+	hOut = GetStdHandle(STD_OUTPUT_HANDLE);
+#endif
+
 #if	I18N
 	/* Initialize internationalization so that messages are in the right language. */
 	setlocale(LC_ALL, "");
-	textdomain(prog_name);
 	bindtextdomain(prog_name, LOCALEDIR);
+	textdomain(prog_name);
 #endif
 
-#if	CYGWIN
+#if	CYGWIN || MINGW
 	dir_path = strdup(dirname_win(argv[0]));	/* set dir_path to this executable's directory */
 #endif
 	/* initialize the global variables */
@@ -215,23 +235,29 @@ char	**argv;
 	}
 #if	READLINE
 	if (readline_enabled) {	/* readline_enabled flag must not change after this */
-#if	!CYGWIN
 		if ((cp = getenv("HOME")) && prog_name) {
+#if	CYGWIN || MINGW
+			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/%s_history", cp, prog_name);
+#else
 			snprintf(history_filename_storage, sizeof(history_filename_storage), "%s/.%s_history", cp, prog_name);
+#endif
 			history_filename = history_filename_storage;
 		}
-#endif
 		using_history();		/* initialize readline history */
 		rl_initialize();		/* initialize readline */
 		stifle_history(500);		/* maximum of 500 entries */
 		rl_inhibit_completion = true;	/* turn off readline file name completion */
-#if	!NO_COLOR && !LIBRARY && !CYGWIN	/* not 100% tested and this might confuse the user with the -c toggle */
+#if	0	/* not 100% tested and this might confuse the user with the -c toggle */
+#if	!WIN32_CONSOLE_COLORS
 		if (!html_flag) {		/* If doing ANSI color: */
 			color_flag = (tigetnum("colors") >= 8);	/* autoset color output mode.  Requires ncurses. */
 		}
 #endif
-#if	!SECURE && !CYGWIN
-		read_history(history_filename);	/* restore readline history of previous session */
+#endif
+#if	!SECURE
+		if (security_level <= 3) {
+			read_history(history_filename);	/* restore readline history of previous session */
+		}
 #endif
 	}
 #endif
@@ -244,8 +270,18 @@ char	**argv;
 	fflush(stdout);
 #if	!SECURE
 	/* read the user options initialization file */
-	if (!test_mode && !demo_mode && !load_rc()) {
-		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n"), prog_name, rc_file);
+	if (security_level <= 3 && !test_mode && !demo_mode && !load_rc()) {
+		fprintf(stderr, _("%s: Error loading set options from \"%s\".\n\n"), prog_name, rc_file);
+		snprintf(prompt_str, sizeof(prompt_str), _("Remove faulty set options startup file (y/n)? "));
+		if (get_yes_no()) {
+			if (unlink(rc_file) == 0) {
+				printf(_("File \"%s\" removed.\n"), rc_file);
+				printf(_("Next startup of Mathomatic should be clean with the defaults set.\n"));
+			} else {
+				perror(rc_file);
+			}
+		}
+		printf("\n");
 	}
 #endif
 	if (wide_flag) {
@@ -263,14 +299,14 @@ char	**argv;
 		color_flag = false;
 	} else if (!quiet_mode && !eoption) {
 		if (color_flag) {
+#if	WIN32_CONSOLE_COLORS
+			printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "WIN32 CONSOLE", bold_colors ? " bold" : "");
+#else
 			printf(_("%s%s color mode enabled"), html_flag ? "HTML" : "ANSI", bold_colors ? " bold" : "");
-			if (!boption) {
-				printf(_("; disable with the -c option or \"set no color\".\n"));
-			} else {
-				printf(".\n");
-			}
+#endif
+			printf(_("; manage by typing \"help color\".\n"));
 		} else {
-			printf(_("Color mode turned off; enable with the -c option or \"set color\".\n"));
+			printf(_("Color mode turned off; manage it by typing \"help color\".\n"));
 		}
 	}
 	fflush(stdout);
@@ -284,6 +320,8 @@ char	**argv;
 			printf(_("Operation aborted.\n"));
 			break;
 		}
+		if (eoption)
+			exit_value = 1;
 	} else {
 		if (!set_signals()) {
 			fprintf(stderr, _("signal(2) setting failed.\n"));
@@ -305,9 +343,16 @@ char	**argv;
 					exit_value = 1;
 				}
 			}
-		}
-#if	!SECURE
-		if (!eoption) {
+		} else {
+#if	SECURE
+			if (!html_flag && !quiet_mode && !test_mode)
+				printf(_("Anything done here is temporary.\n"));
+			if (optind < argc) {
+				warning(_("Command-line argument ignored."));
+			}
+#else
+			if (!html_flag && !quiet_mode && !test_mode)
+				printf(_("Anything done here is temporary, unless it is saved or redirected.\n"));
 			/* read in files specified on the command line, exit if error */
 			for (i = optind; i < argc && argv[i]; i++) {
 				if (strcmp(argv[i], "-") == 0) {
@@ -318,8 +363,8 @@ char	**argv;
 					exit_program(1);
 				}
 			}
-		}
 #endif
+		}
 	}
 	if (!eoption)
 		main_io_loop();		/* main input/output loop */
@@ -358,12 +403,12 @@ set_signals()
 		rv = false;
 	if (signal(SIGINT, inthandler) == SIG_ERR)
 		rv = false;
-	if (signal(SIGQUIT, inthandler) == SIG_ERR)
-		rv = false;
 	if (signal(SIGTERM, exithandler) == SIG_ERR)
 		rv = false;
+#if	!MINGW
 	if (signal(SIGHUP, exithandler) == SIG_ERR)
 		rv = false;
+#endif
 #if	UNIX || CYGWIN
 	if (signal(SIGWINCH, resizehandler) == SIG_ERR)
 		rv = false;
@@ -376,7 +421,7 @@ set_signals()
 	return rv;
 }
 
-#if	CYGWIN
+#if	CYGWIN || MINGW
 /*
  * dirname(3) function for Microsoft Windows.
  * dirname(3) strips the non-directory suffix from a filename.
@@ -419,7 +464,11 @@ load_rc()
 		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, ".mathomaticrc");
 		fp = fopen(rc_file, "r");
 	}
-#if	CYGWIN
+#if	CYGWIN || MINGW
+	if (fp == NULL && cp) {
+		snprintf(rc_file, sizeof(rc_file), "%s/%s", cp, "mathomatic.rc");
+		fp = fopen(rc_file, "r");
+	}
 	if (fp == NULL && dir_path) {
 		snprintf(rc_file, sizeof(rc_file), "%s/%s", dir_path, "mathomatic.rc");
 		fp = fopen(rc_file, "r");
@@ -428,6 +477,9 @@ load_rc()
 	if (fp == NULL) {
 		return true;
 	}
+	if (!quiet_mode && !eoption) {
+		printf(_("Loading startup set options from \"%s\".\n"), rc_file);
+	}
 	while ((cp = fgets(buf, sizeof(buf), fp)) != NULL) {
 		set_error_level(cp);
 		if (!set_options(cp))
@@ -449,7 +501,9 @@ void
 fphandler(sig)
 int	sig;
 {
-/*	warning(_("Floating point exception.")); */
+#if	DEBUG
+	warning("Floating point exception.");
+#endif
 }
 
 /*
@@ -523,13 +577,13 @@ int	exit_value;	/* zero if OK, non-zero indicates error return */
 	if (html_flag) {
 		printf("</pre>\n");
 	}
-#if	READLINE && !SECURE && !CYGWIN
-	if (readline_enabled) {
+#if	READLINE && !SECURE
+	if (readline_enabled && security_level <= 3) {
 		write_history(history_filename);	/* save readline history */
 	}
 #endif
 	if (exit_value == 0 && !quiet_mode && !eoption && !html_flag) {
-		printf(_("Goodbye!\n"));
+		printf(_("ByeBye!! from Mathomatic.\n"));
 	}
 	exit(exit_value);
 }
