@@ -1,22 +1,25 @@
 /*
- * Command line numerical polynomial equation solver using the GNU Scientific Library (GSL).
+ * Command-line numerical polynomial equation solver using the GNU Scientific Library (GSL).
  * GSL is available from: "http://www.gnu.org/software/gsl/".
  *
- * Copyright (C) 2008 George Gesslein II
+ * Copyright (C) 2008-2011 George Gesslein II
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -27,7 +30,7 @@
 
  * or
 
-cc -O -Wall -o roots roots.c -lgsl -lgslcblas
+cc -O3 -Wall -o roots roots.c -lgsl -lgslcblas
 
  * This program nicely solves any degree polynomial
  * with real coefficients by calling the
@@ -35,21 +38,24 @@ cc -O -Wall -o roots roots.c -lgsl -lgslcblas
  * This file is also useful for testing
  * and as an example of using the GSL from C.
  * The results are double precision floating point values
- * that are usually accurate to 14 digits.
- * Solutions are determined by the GSL using
- * balanced-QR reduction of the companion matrix.
+ * that are sometimes accurate to 14 digits.
  * Complex numbers are output if successful.
  * Here is an example of it solving a cubic polynomial:
 
-[~/c/am/misc] roots 1 1 1 1
+$ roots 1 1 1 1 
 The 3 approximate floating point solutions of:
-+1*x^3 +1*x^2 +1*x +1 = 0
++x^3 +x^2 +x +1 = 0
 are:
 
 x = +0 +1*i
 x = +0 -1*i
 x = -1
-[~/c/am/misc] 
+$ 
+
+Try this for a large amount of error:
+roots -1 -1 1 1
+
+Proof the GSL isn't perfect.
 
  */
 
@@ -68,7 +74,8 @@ x = -1
 void	display_root(int i);
 
 char	*prog_name = "roots";		/* name of this program */
-double	*a, *z;		/* input and output arrays, respectively */
+double	*a, *z;				/* input and output arrays, respectively */
+int	precision = DBL_DIG - 1;	/* display precision, it is not useful to set this higher than 15 */
 
 void
 usage()
@@ -78,7 +85,9 @@ usage()
   printf("\nSolves polynomial = 0 when given all real coefficients of the polynomial.\n");
   printf("Double precision floating point math is used, accurate to about 14 digits.\n");
   printf("\nUsage: %s highest-power-coefficient ... constant-term\n", prog_name);
-  printf("\nThe coefficients must be decimal, floating point numbers.\n");
+  printf("\nThe coefficients must be decimal, floating point, real numbers.\n");
+  printf("For example, if 4 real numbers are given, there will be 3 complex number\n");
+  printf("results or \"roots\" that are all valid solutions to polynomial = 0.\n");
 }
 
 int
@@ -119,9 +128,13 @@ main(int argc, char **argv)
   printf("The %d approximate floating point solutions of:\n", highest_power);
   for (i = highest_power; i >= 0; i--) {
     if (a[i]) {
-      printf("%+.*g", DBL_DIG, a[i]);
-      if (i) {
-	printf("*x");
+      if (i && a[i] == 1.0) {
+	printf("+x");
+      } else {
+        printf("%+.*g", precision, a[i]);
+        if (i) {
+	  printf("*x");
+        }
       }
       if (i > 1) {
         printf("^%d", i);
@@ -161,7 +174,7 @@ main(int argc, char **argv)
 void
 display_root(int i)
 {
-  printf("%+.*g", DBL_DIG, z[2*i]);		/* output real part */
+  printf("%+.*g", precision, z[2*i]);		/* output real part */
   if (z[2*i+1])
-    printf(" %+.*g*i", DBL_DIG, z[2*i+1]);	/* output imaginary part */
+    printf(" %+.*g*i", precision, z[2*i+1]);	/* output imaginary part */
 }
