@@ -1,21 +1,24 @@
 /*
  * Expression parsing routines for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -32,7 +35,7 @@ char	*cp;
 {
 	if (cp) {
 		for (; *cp; cp++) {
-			if (isupper(*cp))
+			if (isascii(*cp) && isupper(*cp))
 				*cp = tolower(*cp);
 		}
 	}
@@ -41,12 +44,12 @@ char	*cp;
 /*
  * Display an up arrow pointing to the error, if appropriate.
  * The up arrow is positioned under the input string,
- * followed by the error message.
+ * followed by the error message, which must be a constant string.
  */
 void
 put_up_arrow(cnt, cp)
 int	cnt;	/* position of error, relative to "input_column" */
-char	*cp;	/* error message */
+char	*cp;	/* error message (constant ASCII string) */
 {
 #if	!SILENT
 	int	 i;
@@ -69,9 +72,51 @@ int
 isvarchar(ch)
 int	ch;
 {
+	if (isdigit(ch)) {	/* variable names can never start with a digit */
+		return false;
+	}
 	return(ch == '_' || (ch && strchr(special_variable_characters, ch)) || isalpha(ch));
 }
 
+/*
+ * Return +1 for an opening parenthesis, -1 for a closing parenthesis.
+ * Otherwise, return 0.
+ */
+int
+paren_increment(ch)
+int	ch;
+{
+	switch (ch) {
+	case '(':
+		return 1;
+	case ')':
+		return -1;
+	}
+	return 0;
+}
+
+/*
+ * Return true if character (ch) is the beginning of a Mathomatic operator.
+ */
+int
+is_mathomatic_operator(ch)
+int	ch;
+{
+	switch (ch) {
+	case '!':
+	case '*':
+	case '^':
+	case '/':
+	case '%':
+	case '+':
+	case '-':
+	case '=':
+	case '|':
+		return true;
+	}
+	return false;
+}
+
 /*
  * Parenthesize an operator.
  */
@@ -83,10 +128,14 @@ int		i;	/* location of operator to parenthesize in expression */
 {
 	int	j;
 	int	level;
+	int	skip_negate;
 
-	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1) {
+#if	DEBUG
+	if (i >= (n - 1) || (n & 1) != 1 || (i & 1) != 1 || p1[i].kind != OPERATOR) {
 		error_bug("Internal error in arguments to binary_parenthesize().");
 	}
+#endif
+	skip_negate = (p1[i].token.operatr != NEGATE);
 	level = p1[i].level++;
 	if (p1[i-1].level++ > level) {
 		for (j = i - 2; j >= 0; j--) {
@@ -95,18 +144,23 @@ int		i;	/* location of operator to parenthesize in expression */
 			p1[j].level++;
 		}
 	}
+finish:
 	if (p1[i+1].level++ > level) {
 		for (j = i + 2; j < n; j++) {
 			if (p1[j].level <= level)
 				break;
 			p1[j].level++;
 		}
+	} else if (skip_negate && p1[i+1].level > level && i + 3 < n && p1[i+2].level == level
+	    && p1[i+2].token.operatr == NEGATE) {
+		p1[i+2].level++;
+		i += 2;
+		goto finish;
 	}
 }
 
 /*
- * Handle and remove the special high priority NEGATE operator.
- * To make negate the same priority as times/divide, don't parenthesize here.
+ * Handle and remove the special NEGATE operator.
  */
 void
 handle_negate(equation, np)
@@ -117,10 +171,10 @@ int		*np;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == NEGATE) {
-			/* change negate operator to -1 times the operand: */
-			equation[i].token.operatr = TIMES;
-			/* make negate top priority by parenthesizing it first: */
+			/* parenthesize it first: */
 			binary_parenthesize(equation, *np, i);
+			/* finish changing negate operator to -1.0 times the operand: */
+			equation[i].token.operatr = TIMES;
 		}
 	}
 }
@@ -142,7 +196,7 @@ int		*np;		/* pointer to expression length */
 
 /* Higher priority (precedence) operators need to be parenthesized first: */
 	for (i = 1; i < *np; i += 2) {
-		if (equation[i].token.operatr == FACTORIAL) {
+		if (equation[i].token.operatr >= FACTORIAL) {
 			binary_parenthesize(equation, *np, i);
 		}
 	}
@@ -171,6 +225,7 @@ int		*np;		/* pointer to expression length */
 			break;
 		}
 	}
+	handle_negate(equation, np);	/* Make this the first called function here to make negate highest priority. */
 }
 
 /*
@@ -189,7 +244,8 @@ int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
 int		allow_space;	/* if false, any space characters terminate parsing */
 {
-	int		n = 0;			/* position in equation[] */
+	int		i;
+	int		n = 0, old_n;		/* position in equation[] */
 	int		cur_level = 1;		/* current level of parentheses */
 	int		operand = false;	/* flip-flop between operand and operator */
 	char		*cp_start, *cp1;
@@ -234,6 +290,7 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 			continue;
 		case ' ':
 		case '\t':
+		case '\f':
 			if (allow_space)
 				continue;
 		case '=':
@@ -241,6 +298,14 @@ int		allow_space;	/* if false, any space characters terminate parsing */
 		case '\0':
 		case '\n':
 			goto p_out;	/* terminator encountered */
+		case '\r':
+			continue;
+		case '\033':
+			if (cp[1] == '[' || cp[1] == 'O') {
+				error(_("Cursor or function key string encountered, unable to correct."));
+				return(NULL);
+			}
+			continue;
 		}
 		operand = !operand;
 		switch (*cp) {
@@ -419,6 +484,48 @@ parse_power:
 			n++;
 			cp--;
 			break;
+		case '#':
+			if (!operand) {
+				goto syntax_error;
+			}
+			cp++;
+			switch (*cp) {
+			case '+':
+			case '-':
+				i = decstrtol(cp, &cp1);
+				i = cur_equation + i;
+				break;
+			default:
+				i = decstrtol(cp, &cp1) - 1;
+				break;
+			}
+			if (cp == cp1) {
+				put_up_arrow(cp - cp_start, _("Error parsing equation space number after #."));
+				return NULL;
+			}
+			if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
+				put_up_arrow(cp - cp_start, _("No expression available in # specified equation space."));
+				return NULL;
+			}
+			cp = cp1 - 1;
+			old_n = n;
+			if (n_rhs[i]) {
+				n += n_rhs[i];
+				if (n > n_tokens) {
+					error_huge();
+				}
+				blt(&equation[old_n], rhs[i], n_rhs[i] * sizeof(token_type));
+			} else {
+				n += n_lhs[i];
+				if (n > n_tokens) {
+					error_huge();
+				}
+				blt(&equation[old_n], lhs[i], n_lhs[i] * sizeof(token_type));
+			}
+			for (; old_n < n; old_n++) {
+				equation[old_n].level += cur_level;
+			}
+			break;
 		default:
 			if (!isvarchar(*cp)) {
 				put_up_arrow(cp - cp_start, _("Unrecognized character."));
@@ -432,7 +539,12 @@ parse_power:
 				n++;
 			}
 			cp1 = cp;
-			if (strncasecmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
+			if (strncasecmp(cp, "inf", strlen("inf")) == 0
+			    && !isvarchar(cp[strlen("inf")])) {
+				equation[n].kind = CONSTANT;
+				equation[n].token.constant = INFINITY;	/* the infinity constant */
+				cp += strlen("inf");
+			} else if (strncasecmp(cp, INFINITY_NAME, strlen(INFINITY_NAME)) == 0
 			    && !isvarchar(cp[strlen(INFINITY_NAME)])) {
 				equation[n].kind = CONSTANT;
 				equation[n].token.constant = INFINITY;	/* the infinity constant */
@@ -466,7 +578,6 @@ p_out:
 		cp++;
 	*np = n;
 	if (n) {
-		handle_negate(equation, np);
 		give_priority(equation, np);
 		organize(equation, np);
 	}
@@ -491,13 +602,6 @@ parse_equation(n, cp)
 int	n;	/* equation space number */
 char	*cp;	/* pointer to the beginning of the equation character string */
 {
-#if	0
-	if (cp == NULL)
-		return NULL;
-	if (!case_sensitive_flag) {
-		str_tolower(cp);
-	}
-#endif
 	if ((cp = parse_expr(lhs[n], &n_lhs[n], cp, true)) != NULL) {
 		if ((cp = parse_expr(rhs[n], &n_rhs[n], cp, true)) != NULL) {
 			if (!extra_characters(cp))
@@ -511,7 +615,7 @@ char	*cp;	/* pointer to the beginning of the equation character string */
 
 /*
  * Parse an expression (not an equation) string, with equation space pull
- * if the string starts with "#" followed by an equation number.
+ * if the string contains "#" followed by an equation number.
  *
  * Returns the new string position, or NULL if error.
  */
@@ -520,54 +624,13 @@ parse_expr(equation, np, cp, allow_space)
 token_type	*equation;	/* where the parsed expression is stored (equation side) */
 int		*np;		/* pointer to the returned parsed expression length */
 char		*cp;		/* string to parse */
-int		allow_space;
+int		allow_space;	/* if true, allow and ignore space characters; if false, space means terminate parsing */
 {
-	int	i;
-	char	*cp1, *cp2, *cp_start;
-
 	if (cp == NULL)
 		return NULL;
-	cp_start = cp;
 	if (!case_sensitive_flag) {
 		str_tolower(cp);
 	}
-	cp1 = skip_space(cp);
-	if (*cp1 == '#') {
-		cp1++;
-		switch (*cp1) {
-		case '+':
-		case '-':
-			i = decstrtol(cp1, &cp2);
-			i = cur_equation + i;
-			break;
-		default:
-			i = decstrtol(cp1, &cp2) - 1;
-			break;
-		}
-		if (cp1 == cp2) {
-			put_up_arrow(cp1 - cp_start, _("Error parsing equation space number after #."));
-			return NULL;
-		}
-		if (i < 0 || i >= n_equations || n_lhs[i] <= 0) {
-			put_up_arrow(cp1 - cp_start, _("No expression available in specified equation space."));
-			return NULL;
-		}
-		if (n_rhs[i]) {
-			blt(equation, rhs[i], n_rhs[i] * sizeof(token_type));
-			*np = n_rhs[i];
-		} else {
-			blt(equation, lhs[i], n_lhs[i] * sizeof(token_type));
-			*np = n_lhs[i];
-		}
-#if	!SILENT
-		list_proc(equation, *np, false);
-		fprintf(gfp, "\n");
-#endif
-		while (*cp2 == '=')
-			cp2++;
-		input_column += (cp2 - cp_start);
-		return cp2;
-	}
 	cp = parse_section(equation, np, cp, allow_space);
 	return cp;
 }
@@ -591,34 +654,44 @@ char	*cp;
 	char	buf[MAX_VAR_LEN+1];
 	char	*cp1;
 	int	len;
+	int	level;		/* parentheses level */
 	int	(*strcmpfunc)();
 
-/*	*vp = V_NULL; */
 	if (case_sensitive_flag) {
 		strcmpfunc = strcmp;
 	} else {
 		strcmpfunc = strcasecmp;
 	}
-	if (!isvarchar(*cp)) {
+	if (!isvarchar(*cp) || paren_increment(*cp) < 0) {
 		error(_("Invalid variable."));
 		return(NULL);	/* variable name must start with a valid variable character */
 	}
-	for (cp1 = cp, i = 0; *cp1;) {
-		if (!isvarchar(*cp1)) {
+	for (level = 0, cp1 = cp, i = 0; *cp1;) {
+		if (level <= 0 && !isvarchar(*cp1)) {
 			break;
 		}
+		j = paren_increment(*cp1);
+		level += j;
+		if (level < 0)
+			break;
 		if (i >= MAX_VAR_LEN) {
 			error(_("Variable name too long."));
 			return(NULL);
 		}
 		buf[i++] = *cp1++;
+		if (j == -1 && level <= 0)
+			break;
 	}
 	buf[i] = '\0';
+	if (level > 0) {
+		error(_("Unmatched parenthesis: missing )"));
+		return(NULL);
+	}
 
 	if (strcasecmp(buf, NAN_NAME) == 0) {
 		warning(_("Attempt to enter NaN (Not a Number); Converted to variable."));
 	}
-	if (strcasecmp(buf, INFINITY_NAME) == 0) {
+	if (strcasecmp(buf, "inf") == 0 || strcasecmp(buf, INFINITY_NAME) == 0) {
 		error(_("Infinity cannot be used as a variable."));
 		return(NULL);
 	} else if ((*strcmpfunc)(buf, "sign") == 0) {
@@ -636,17 +709,31 @@ char	*cp;
 			*vp = V_PI;
 			return(cp + 3);
 		}
-		for (cp1 = cp, i = 0; *cp1;) {
-			if (!isvarchar(*cp1) && !isdigit(*cp1)) {
+		for (level = 0, cp1 = cp, i = 0; *cp1;) {
+			if (level <= 0 && !isvarchar(*cp1) && !isdigit(*cp1)) {
 				break;
 			}
+			j = paren_increment(*cp1);
+			level += j;
+			if (level < 0)
+				break;
 			if (i >= MAX_VAR_LEN) {
 				error(_("Variable name too long."));
 				return(NULL);
 			}
 			buf[i++] = *cp1++;
+			if (j == -1 && level <= 0)
+				break;
+		}
+		if (i <= 0) {
+			error(_("Empty variable name parsed!"));
+			return(NULL);
 		}
 		buf[i] = '\0';
+		if (level > 0) {
+			error(_("Unmatched parenthesis: missing )"));
+			return(NULL);
+		}
 		if ((*strcmpfunc)(buf, "i") == 0) {
 			*vp = IMAGINARY;
 			return(cp1);
@@ -702,6 +789,8 @@ char	*cp;
 			sign_array[j+1] = true;
 		}
 		vtmp += ((long) (j + 1)) << VAR_SHIFT;
+	} else if (vtmp == SIGN) {
+		sign_array[0] = true;
 	}
 	*vp = vtmp;
 	return cp1;
@@ -736,10 +825,10 @@ char	*cp;	/* input string */
 	int	len;
 
 	point_flag = true;
-/* handle comments, newlines, and the DOS EOF character (control-Z) by truncating the string where found */
+/* handle comments, line breaks, and the DOS EOF character (control-Z) by truncating the string where found */
 	cp1 = cp;
 	while ((cp1 = strpbrk(cp1, ";\n\r\032"))) {
-		if (cp1 != cp && *(cp1 - 1) == '\\') {
+		if (cp1 > cp && *(cp1 - 1) == '\\') {
 			if (*cp1 == ';') {
 /* skip backslash escaped semicolon */
 				point_flag = false;
@@ -810,7 +899,7 @@ int		*np;
 
 #ifndef	my_strlcpy
 /*
- * A very efficient strlcpy().
+ * A very efficient strlcpy(), which isn't available in some distributions, hence this code.
  *
  * Copy up to (n - 1) characters from string in src
  * to dest and null-terminate the result.
@@ -825,9 +914,11 @@ int	n;
 	int	len, len_src;
 
 	len_src = strlen(src);
-	len = min(n - 1, len_src);
-	memmove(dest, src, len);
-	dest[len] = '\0';
+	if (n > 0) {
+		len = min(n - 1, len_src);
+		memmove(dest, src, len);
+		dest[len] = '\0';
+	}
 	return len_src;
 }
 #endif
