@@ -18,21 +18,24 @@
  * They are not separated out from the main variable of the polynomial.
  * Maximum simplification of all expressions is not possible unless everything is generalized.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -55,9 +58,15 @@ int		n_quotient;			/* length of expression in quotient[] */
 token_type	gcd_divisor[DIVISOR_SIZE];	/* static expression storage area for polynomial GCD routine */
 int		len_d;				/* length of expression in gcd_divisor[] */
 
-static int	do_gcd(), pf_recurse(), pf_sub(), mod_recurse(), polydiv_recurse(), pdiv_recurse(), poly_div_sub();
-static int	find_highest_count();
-static void	save_factors();
+static int pf_recurse(token_type *equation, int *np, int loc, int level, int do_repeat);
+static int pf_sub(token_type *equation, int *np, int loc, int len, int level, int do_repeat);
+static int save_factors(token_type *equation, int *np, int loc1, int len, int level);
+static int do_gcd(long *vp);
+static int mod_recurse(token_type *equation, int *np, int loc, int level);
+static int polydiv_recurse(token_type *equation, int *np, int loc, int level);
+static int pdiv_recurse(token_type *equation, int *np, int loc, int level, int code);
+static int poly_div_sub(token_type *d1, int len1, token_type *d2, int len2, long *vp);
+static int find_highest_count(token_type *p1, int n1, token_type *p2, int n2, long *vp1);
 
 /*
  * Compare function for qsort(3).
@@ -66,6 +75,13 @@ static int
 vcmp(p1, p2)
 sort_type	*p1, *p2;
 {
+	if (p2->count == p1->count) {
+		if (p1->v < p2->v)
+			return -1;
+		if (p1->v == p2->v)
+			return 0;
+		return 1;
+	}
 	return(p2->count - p1->count);
 }
 
@@ -75,7 +91,7 @@ sort_type	*p1, *p2;
  * The coefficient (c) and exponent (d) may be any expression,
  * as long as it doesn't contain the variable v.
  */
-static int
+int
 poly_in_v_sub(p1, n, v, allow_divides)
 token_type	*p1;		/* expression pointer */
 int		n;		/* expression length */
@@ -215,7 +231,7 @@ int		do_repeat;
  * It can't factor everything, but it usually can factor polynomials
  * if it will make the expression size smaller.
  *
- * Return true if expression was modified (factored).
+ * Return true if equation side was modified (factored).
  */
 static int
 pf_sub(equation, np, loc, len, level, do_repeat)
@@ -292,7 +308,7 @@ int		do_repeat;	/* factor repeated factors flag */
 		}
 		side_debug(3, &equation[loc1], len);
 		side_debug(3, trhs, n_trhs);
-/* find a valid polynomial base variable "v": */
+/* Find a valid polynomial base variable "v": */
 		cnt = -1;
 		if (v) {
 			if (!poly_in_v(trhs, n_trhs, v, true)) {
@@ -312,13 +328,13 @@ int		do_repeat;	/* factor repeated factors flag */
 				cnt = va[i].count;
 			}
 		}
-		if (cnt <= 1)
+		if (cnt <= 1)	/* A polynomial requires 2 or more instances of every normal variable to be factorable. */
 			goto skip_factor;
 		if (v == 0) {
 #if	1
 			goto skip_factor;
 #else
-/* no polynomial variables found, try differentiating with respect to each variable until one works */
+/* no polynomial variables found, try differentiating with respect to each normal variable until one works */
 			for (i = 0; i < vc; i++) {
 				if ((va[i].v & VAR_MASK) <= SIGN) {
 					continue;
@@ -339,7 +355,7 @@ int		do_repeat;	/* factor repeated factors flag */
 			blt(tlhs, trhs, n_trhs * sizeof(token_type));
 			n_tlhs = n_trhs;
 			if (!differentiate(tlhs, &n_tlhs, v)) {
-				goto skip_factor;
+				break;
 			}
 		}
 #if	!SILENT
@@ -362,11 +378,12 @@ int		do_repeat;	/* factor repeated factors flag */
 		tlhs[n_tlhs].token.variable = v;
 		n_tlhs++;
 		uf_simp(tlhs, &n_tlhs);
-		if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v) == 0)
+		if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, v) <= 0)
 			break;
-		if (!level1_plus_count(tlhs, n_tlhs))
+		if (level1_plus_count(tlhs, n_tlhs) == 0)
+			break;
+		if (!save_factors(equation, np, loc1, len, level))
 			break;
-		save_factors(equation, np, loc1, len, level);
 		loc1 += n_tlhs + 1;
 		len = n_trhs;
 		switch (count) {
@@ -385,10 +402,14 @@ int		do_repeat;	/* factor repeated factors flag */
 	if (!modified) {
 		last_v = 0;
 next_v:
-		p1 = &equation[loc];
+		p1 = &equation[loc1];
 		blt(trhs, p1, len * sizeof(token_type));
 		n_trhs = len;
-		uf_simp(trhs, &n_trhs);
+		uf_simp_no_repeat(trhs, &n_trhs);
+		if (level1_plus_count(trhs, n_trhs) < 2) {
+/* must be at least 2 level 1 additive operators to be factorable */
+			goto skip_factor;
+		}
 		for (;;) {
 			v = -1;
 			for (i = 0; i < len; i += 2) {
@@ -425,18 +446,21 @@ next_v:
 				fprintf(gfp, "\n");
 			}
 #endif
-			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) == 0)
+			if (poly_gcd(&equation[loc1], len, tlhs, n_tlhs, 0L) <= 0)
 				goto next_v;
-			if (!level1_plus_count(tlhs, n_tlhs))
+			if (level1_plus_count(tlhs, n_tlhs) == 0)
 				goto next_v;
-			debug_string(1, "Symbolic polynomial factored.");
-			save_factors(equation, np, loc1, len, level);
-			loc1 += n_tlhs + 1;
-			len = n_trhs;
-			len_first = n_tlhs;
-			loc2 = loc1;
+			if (!symbolic_modified) {
+				debug_string(1, "Symbolic polynomial factored.");
+			} else {
+				debug_string(1, "Found another symbolic factor.");
+			}
+			if (!save_factors(equation, np, loc1, len, level))
+				break;
+			len = n_tlhs;
 			symbolic_modified = true;
-			break;
+			last_v = 0;
+			goto next_v;
 		}
 	}
 skip_factor:
@@ -451,23 +475,24 @@ skip_factor:
 			loct = loc;
 			lent = len_first;
 		}
-		if (poly_gcd(&equation[loc1], len, &equation[loct], lent, v)) {
-			save_factors(equation, np, loc1, len, level);
-			loc1 += n_tlhs + 1;
-			len = n_trhs;
+		if (poly_gcd(&equation[loc1], len, &equation[loct], lent, v) > 0) {
+			if (save_factors(equation, np, loc1, len, level)) {
+				loc1 += n_tlhs + 1;
+				len = n_trhs;
+			}
 		}
 		if (len2) {
 			loc1 = loc2;
 			len = len2;
 		}
-		if (poly_gcd(&equation[loc], len_first, &equation[loc1], len, 0L)) {
+		if (poly_gcd(&equation[loc], len_first, &equation[loc1], len, 0L) > 0) {
 			save_factors(equation, np, loc, len_first, level);
 		}
 	}
 	return(modified || symbolic_modified);
 }
 
-static void
+static int
 save_factors(equation, np, loc1, len, level)
 token_type	*equation;
 int		*np, loc1, len, level;
@@ -475,8 +500,10 @@ int		*np, loc1, len, level;
 	int	i, j;
 
 	i = n_tlhs + 1 + n_trhs;
+	if (i > (len * 3))
+		return false;
 	if ((*np + (i - len)) > n_tokens)
-		error_huge();
+		return false;
 	blt(&equation[loc1+i], &equation[loc1+len], (*np - (loc1 + len)) * sizeof(token_type));
 	*np += i - len;
 	blt(&equation[loc1], tlhs, n_tlhs * sizeof(token_type));
@@ -489,10 +516,12 @@ int		*np, loc1, len, level;
 	i += n_trhs;
 	for (j = loc1; j < i; j++)
 		equation[j].level += level;
+	return true;
 }
 
 /*
- * Remove level 1 trivial factors and divides from tlhs.
+ * Remove level 1 trivial factors and divides from tlhs[].
+ * Makes it a simpler factor that looks a lot nicer and is easier to work with.
  *
  * Return true if result is a level 1 additive expression.
  * If this returns false, nothing is removed.
@@ -507,7 +536,7 @@ remove_factors()
 	debug_string(3, "Entering remove_factors() with:");
 	side_debug(3, tlhs, n_tlhs);
 	do {
-		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 4 /* was 0 */);
+		simp_ssub(tlhs, &n_tlhs, 0L, 1.0, false, true, 4);
 	} while (uf_power(tlhs, &n_tlhs));
 	for (i = 1, j = 0, k = 0;; i += 2) {
 		if (i >= n_tlhs) {
@@ -561,14 +590,16 @@ remove_factors()
 
 /*
  * This is the Euclidean GCD algorithm applied to polynomials.
- * It needs to be made multivariate.
+ * It needs to be made multivariate by making it recursive.
  *
- * Return the number of iterations (divisions), if successful.
- * Return 0 on failure.
+ * Return the number of iterations (divisions), if successful,
+ * with the polynomial GCD result in gcd_divisor[] and len_d.
+ * Else return 0 or negative on failure.  If 0, might work if operands switched,
+ * If a negative number, switching operands won't work either.
  */
 static int
 do_gcd(vp)
-long		*vp;
+long		*vp;	/* polynomial base variable pointer */
 {
 	int	i;
 	int	count;
@@ -577,12 +608,14 @@ long		*vp;
 		switch (poly_div(trhs, n_trhs, gcd_divisor, len_d, vp)) {
 		case 0:
 			/* divide failed */
-			return 0;
+			return(1 - count);
 		case 2:
 			/* Total success!  Remainder is zero. */
+			debug_string(2, "Found raw polynomial GCD:");
+			side_debug(2, gcd_divisor, len_d);
 			return count;
 		}
-		/* swap trhs[] (remainder) and gcd_divisor[] */
+		/* Do the Euclidean shuffle: swap trhs[] (remainder) and gcd_divisor[] */
 		if (len_d > n_tokens || n_trhs > DIVISOR_SIZE)
 			return 0;
 		blt(scratch, trhs, n_trhs * sizeof(token_type));
@@ -596,27 +629,37 @@ long		*vp;
 }
 
 /*
- * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
- * This GCD routine is used for polynomial factoring, etc.
+ * Compute the simplified and normalized polynomial Greatest Common Divisor
+ * of the expressions in "larger" and "smaller".
+ * This polynomial GCD routine is used for polynomial factoring, etc.
  *
- * Return true if successful.
+ * Return a positive integer if successful.
  * Return the GCD in trhs[].
  * Return larger/GCD in tlhs[].
+ * The results are unfactored and simplified.
  */
 int
 poly_gcd(larger, llen, smaller, slen, v)
 token_type	*larger;	/* larger polynomial */
+int		llen;		/* larger polynomial length */
 token_type	*smaller;	/* smaller polynomial */
-int		llen, slen;
-long		v;
+int		slen;		/* smaller polynomial length */
+long		v;		/* polynomial base variable */
 {
 	int		count;
 
+	debug_string(3, "Entering poly_gcd():");
+	side_debug(3, larger, llen);
+	side_debug(3, smaller, slen);
 	if (llen > n_tokens || slen > min(ARR_CNT(gcd_divisor), n_tokens))
 		return 0;
-	blt(trhs, larger, llen * sizeof(token_type));
+	if (trhs != larger) {
+		blt(trhs, larger, llen * sizeof(token_type));
+	}
 	n_trhs = llen;
-	blt(tlhs, smaller, slen * sizeof(token_type));
+	if (tlhs != smaller) {
+		blt(tlhs, smaller, slen * sizeof(token_type));
+	}
 	n_tlhs = slen;
 	if (!remove_factors())
 		return 0;
@@ -625,7 +668,7 @@ long		v;
 	blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 	len_d = n_tlhs;
 	count = do_gcd(&v);
-	if (count == 0)
+	if (count <= 0)
 		return 0;
 	if (count > 1) {
 		if (len_d > n_tokens)
@@ -655,23 +698,47 @@ long		v;
 
 /*
  * Compute the polynomial Greatest Common Divisor of the expressions in "larger" and "smaller".
- * This GCD routine is used by division simplifiers.
+ * This polynomial GCD routine is used by the division simplifiers.
  *
- * Return true if successful.
+ * Return a positive integer if successful.
  * Return larger/GCD in tlhs[].
  * Return smaller/GCD in trhs[].
  */
 int
-poly2_gcd(larger, llen, smaller, slen, v)
+poly2_gcd(larger, llen, smaller, slen, v, require_additive)
 token_type	*larger;	/* larger polynomial */
+int		llen;		/* larger polynomial length */
 token_type	*smaller;	/* smaller polynomial */
-int		llen, slen;
-long		v;
+int		slen;		/* smaller polynomial length */
+long		v;		/* polynomial base variable */
+int		require_additive;	/* require the GCD to contain addition or subtraction */
 {
 	int		i;
-	jmp_buf		save_save;
 	int		count;
+#if	0
+	jmp_buf		save_save;
+#endif
 
+	if (require_additive) {
+/* Require an additive operator in both polynomials to continue. */
+		count = 0;
+		for (i = 1; i < llen; i += 2) {
+			if (larger[i].token.operatr == PLUS || larger[i].token.operatr == MINUS) {
+				count++;
+				break;
+			}
+		}
+		if (count == 0)
+			return 0;
+		count = 0;
+		for (i = 1; i < slen; i += 2) {
+			if (smaller[i].token.operatr == PLUS || smaller[i].token.operatr == MINUS) {
+				count++;
+			}
+		}
+		if (count == 0 /* || count > 200 */)
+			return 0;
+	}
 	debug_string(3, "Entering poly2_gcd():");
 	side_debug(3, larger, llen);
 	side_debug(3, smaller, slen);
@@ -681,29 +748,32 @@ long		v;
 	n_trhs = llen;
 	blt(tlhs, smaller, slen * sizeof(token_type));
 	n_tlhs = slen;
-#if	1
-	blt(save_save, jmp_save, sizeof(jmp_save));
-	if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
-		blt(jmp_save, save_save, sizeof(jmp_save));
-		if (i == 13) {	/* critical error code */
-			longjmp(jmp_save, i);
+#if	0
+	if (require_additive) {
+/* Require a level 1 additive operator in the divisor. */
+		blt(save_save, jmp_save, sizeof(jmp_save));
+		if ((i = setjmp(jmp_save)) != 0) {	/* trap errors */
+			blt(jmp_save, save_save, sizeof(jmp_save));
+			if (i == 13) {	/* critical error code */
+				longjmp(jmp_save, i);
+			}
+			return 0;
 		}
-		return 0;
+		uf_simp(tlhs, &n_tlhs);
+		blt(jmp_save, save_save, sizeof(jmp_save));
+		if (level1_plus_count(tlhs, n_tlhs) == 0)
+			return 0;
 	}
-	uf_simp(tlhs, &n_tlhs);
-	blt(jmp_save, save_save, sizeof(jmp_save));
 #endif
-	if (!level1_plus_count(tlhs, n_tlhs))
-		return 0;
 	if (n_tlhs > ARR_CNT(gcd_divisor))
 		return 0;
 	blt(gcd_divisor, tlhs, n_tlhs * sizeof(token_type));
 	len_d = n_tlhs;
 	count = do_gcd(&v);
-	if (count == 0)
-		return 0;
+	if (count <= 0)
+		return count;
 	if (count > 1) {
-		if (!level1_plus_count(gcd_divisor, len_d))
+		if (require_additive && level1_plus_count(gcd_divisor, len_d) == 0)
 			return 0;
 		if (poly_div(smaller, slen, gcd_divisor, len_d, &v) != 2) {
 			debug_string(1, "Polynomial GCD found, but smaller divide failed in poly2_gcd().");
@@ -798,7 +868,7 @@ int		n;	/* length of expression */
  * Using "integer" variables will allow more simplification here.
  * Globals tlhs[] and trhs[] are wiped out by the polynomial division.
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 mod_simp(equation, np)
@@ -819,7 +889,6 @@ int		*np, loc, level;
 	int	op, last_op2;
 	int	len1, len2, len3;
 	int	diff_sign;
-	long	v;		/* Mathomatic variable */
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -940,8 +1009,7 @@ int		*np, loc, level;
 			/* Remove integer*n multiples in x for x%n by doing */
 			/* polynomial division x/n and replacing with remainder%n. */
 			/* Globals tlhs[] and trhs[] are wiped out by the polynomial division here. */
-			v = 0;
-			if (poly_div(&equation[j], len2, &equation[i+1], len1, &v)) {
+			if (poly_div(&equation[j], len2, &equation[i+1], len1, NULL)) {
 				uf_pplus(tlhs, &n_tlhs);	/* so integer%(integer^integer) isn't simplified to 0 */
 				if (is_integer_expr(tlhs, n_tlhs)) {
 					if ((*np + (n_trhs - len2)) > n_tokens)
@@ -985,6 +1053,7 @@ int		*np, loc, level;
 	int	i, j, k;
 	int	last_op2;
 	int	len1, len2;
+	int	rv;
 
 	for (i = loc; i < *np && equation[i].level >= level;) {
 		if (equation[i].level > level) {
@@ -998,6 +1067,10 @@ int		*np, loc, level;
 	}
 start:
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
+#if	DEBUG
+		if (equation[i].kind != OPERATOR)
+			error_bug("Bug in poly_gcd_simp().");
+#endif
 		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
 			for (k = i + 2;; k += 2) {
 				if (k >= *np || equation[k].level <= level)
@@ -1010,8 +1083,14 @@ start:
 					last_op2 = equation[j].token.operatr;
 					continue;
 				}
-				if (last_op2 == DIVIDE) {
+				switch (last_op2) {
+				case DIVIDE:
 					continue;
+				case 0:
+				case TIMES:
+					break;
+				default:
+					error_bug("Expression is corrupt in poly_gcd_simp().");
 				}
 				last_op2 = DIVIDE;
 				for (k = j + 1;; k += 2) {
@@ -1019,7 +1098,7 @@ start:
 						break;
 				}
 				len2 = k - j;
-				if (poly2_gcd(&equation[i+1], len1, &equation[j], len2, 0L)) {
+				if ((rv = poly2_gcd(&equation[i+1], len1, &equation[j], len2, 0L, true)) > 0) {
 store_code:
 					for (k = 0; k < n_tlhs; k++)
 						tlhs[k].level += level;
@@ -1040,7 +1119,7 @@ store_code:
 					modified = true;
 					goto start;
 				}
-				if (poly2_gcd(&equation[j], len2, &equation[i+1], len1, 0L)) {
+				if (rv == 0 && poly2_gcd(&equation[j], len2, &equation[i+1], len1, 0L, true) > 0) {
 					k = j - 1;
 					j = i + 1;
 					i = k;
@@ -1058,6 +1137,12 @@ store_code:
 /*
  * This routine is a division simplifier for equation sides.
  * Check for divides and do polynomial and smart division.
+ * For example: (c - b + a)/(b - a) will be converted to (c/(b - a)) - 1.
+ * Will only make expressions with algebraic fractions smaller,
+ * otherwise it would go into an infinite loop.
+ * An expression with less operators is judged to be smaller.
+ * Complex fractions (fractions within fractions) are sometimes
+ * created by this, especially if quick_flag is false.
  *
  * Return true if expression was simplified.
  */
@@ -1065,16 +1150,18 @@ int
 div_remainder(equation, np, poly_flag, quick_flag)
 token_type	*equation;
 int		*np;
-int		poly_flag;
-int		quick_flag;
+int		poly_flag;	/* if true, try polynomial division first, then smart division */
+int		quick_flag;	/* if true, keep algebraic fractions simpler */
 {
 	int	rv = false;
 
+	debug_string(3, "Entering div_remainder().");
 	if (quick_flag)
 		group_proc(equation, np);
 	rv = pdiv_recurse(equation, np, 0, 1, poly_flag);
 	if (quick_flag)
 		organize(equation, np);
+	debug_string(3, "Leaving div_remainder().");
 	return rv;
 }
 
@@ -1087,155 +1174,153 @@ int		*np, loc, level, code;
 	int	i, j, k;
 	int	op, op2, last_op2;
 	int	len1, len2, real_len1;
-	long	v;
 	int	rv = 0;
 	int	flag, power_flag, zero_remainder;
 
 	for (i = loc + 1; i < *np && equation[i].level >= level; i += 2) {
-		if (equation[i].level == level && equation[i].token.operatr == DIVIDE) {
-			for (k = i + 2;; k += 2) {
-				if (k >= *np || equation[k].level <= level)
-					break;
+		if (!(equation[i].level == level && equation[i].token.operatr == DIVIDE))
+			continue;
+		for (k = i + 2;; k += 2) {
+			if (k >= *np || equation[k].level <= level)
+				break;
+		}
+		len1 = real_len1 = k - (i + 1);
+		last_op2 = 0;
+		for (j = loc; j < *np && equation[j].level >= level; j++) {
+			if (equation[j].level == level && equation[j].kind == OPERATOR) {
+				last_op2 = equation[j].token.operatr;
+				continue;
 			}
-			len1 = real_len1 = k - (i + 1);
-			last_op2 = 0;
-			for (j = loc; j < *np && equation[j].level >= level; j++) {
-				if (equation[j].level == level && equation[j].kind == OPERATOR) {
-					last_op2 = equation[j].token.operatr;
-					continue;
+			if (last_op2 == DIVIDE) {
+				continue;
+			}
+			last_op2 = DIVIDE;
+			op = 0;
+			for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == (level + 1)) {
+					op = equation[k].token.operatr;
 				}
-				if (last_op2 == DIVIDE) {
-					continue;
-				}
-				last_op2 = DIVIDE;
-				op = 0;
-				for (k = j + 1; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == (level + 1)) {
-						op = equation[k].token.operatr;
+			}
+			if (op != PLUS && op != MINUS) {
+				continue;
+			}
+			len2 = k - j;
+			flag = code;
+			power_flag = false;
+			op = 0;
+			op2 = 0;
+			for (k = i + 2; k < *np && equation[k].level > level; k += 2) {
+				if (equation[k].level == level + 3) {
+					switch (equation[k].token.operatr) {
+					case PLUS:
+					case MINUS:
+						op2 = PLUS;
 					}
-				}
-				if (op != PLUS && op != MINUS) {
-					continue;
-				}
-				len2 = k - j;
-				flag = code;
-				power_flag = false;
-				op = 0;
-				op2 = 0;
-				for (k = i + 2; k < *np && equation[k].level > level; k += 2) {
-					if (equation[k].level == level + 3) {
-						switch (equation[k].token.operatr) {
-						case PLUS:
-						case MINUS:
-							op2 = PLUS;
-						}
-					} else if (equation[k].level == level + 2) {
-						op = equation[k].token.operatr;
-					} else if (equation[k].level == level + 1) {
-						if (equation[k].token.operatr == POWER
-						    && (op == PLUS || op == MINUS || (op == TIMES && op2 == PLUS))) {
-							power_flag = true;
-							len1 = k - (i + 1);
-						}
-						break;
+				} else if (equation[k].level == level + 2) {
+					op = equation[k].token.operatr;
+				} else if (equation[k].level == level + 1) {
+					if (equation[k].token.operatr == POWER
+					    && (op == PLUS || op == MINUS || (op == TIMES && op2 == PLUS))) {
+						power_flag = true;
+						len1 = k - (i + 1);
 					}
+					break;
 				}
+			}
 next_thingy:
-				if (!power_flag) {
-					len1 = real_len1;
-				}
-				v = 0;
-				if (flag || power_flag) {
-					rv = poly_div(&equation[j], len2, &equation[i+1], len1, &v);
-				} else {
-					rv = smart_div(&equation[j], len2, &equation[i+1], len1);
-				}
-				zero_remainder = (rv > 0 && REMAINDER_IS_ZERO());
-				if (power_flag && !zero_remainder) {
-					rv = 0;
-				}
-				if (rv > 0) {
-					if ((n_tlhs + 2 + n_trhs + len1) > n_tokens)
-						error_huge();
-					for (k = 0; k < n_tlhs; k++)
-						tlhs[k].level++;
-					tlhs[n_tlhs].level = 1;
-					tlhs[n_tlhs].kind = OPERATOR;
-					tlhs[n_tlhs].token.operatr = PLUS;
-					n_tlhs++;
-					for (k = 0; k < n_trhs; k++)
-						trhs[k].level += 2;
-					blt(&tlhs[n_tlhs], trhs, n_trhs * sizeof(token_type));
-					n_tlhs += n_trhs;
-					tlhs[n_tlhs].level = 2;
-					tlhs[n_tlhs].kind = OPERATOR;
-					tlhs[n_tlhs].token.operatr = DIVIDE;
-					n_tlhs++;
-					k = n_tlhs;
-					blt(&tlhs[n_tlhs], &equation[i+1], len1 * sizeof(token_type));
-					n_tlhs += len1;
-					for (; k < n_tlhs; k++)
-						tlhs[k].level += 2;
-					side_debug(3, &equation[j], len2);
-					side_debug(3, &equation[i+1], len1);
-					simpb_side(tlhs, &n_tlhs, false, true, 3);	/* parameters should match what's used in simpa_side() */
-					side_debug(3, tlhs, n_tlhs);
-					if (power_flag) {
-						k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
-					} else {
-						k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
-					}
-					if (k) {
-						for (k = 0; k < n_tlhs; k++)
-							tlhs[k].level += level;
-						if (power_flag) {
-							if ((*np - len2 + n_tlhs + 2) > n_tokens)
-								error_huge();
-							for (k = i + 2 + len1; k <= i + real_len1; k++) {
-								equation[k].level++;
-							}
-							blt(&equation[i+real_len1+3], &equation[k], (*np - k) * sizeof(token_type));
-							*np += 2;
-							equation[k].level = level + 2;
-							equation[k].kind = OPERATOR;
-							equation[k].token.operatr = MINUS;
-							k++;
-							equation[k].level = level + 2;
-							equation[k].kind = CONSTANT;
-							equation[k].token.constant = 1.0;
-							if (i < j) {
-								j += 2;
-							}
-						} else {
-							if ((*np - (len1 + 1 + len2) + n_tlhs) > n_tokens)
-								error_huge();
-							blt(&equation[i], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
-							*np -= len1 + 1;
-							if (i < j) {
-								j -= len1 + 1;
-							}
-						}
-						blt(&equation[j+n_tlhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
-						*np -= len2 - n_tlhs;
-						blt(&equation[j], tlhs, n_tlhs * sizeof(token_type));
-						if (flag || power_flag) {
-							debug_string(1, _("Polynomial division successful."));
-						} else {
-							debug_string(1, _("Smart division successful."));
-						}
-						side_debug(3, equation, *np);
-						return true;
-					}
-				}
+			if (!power_flag) {
+				len1 = real_len1;
+			}
+			if (flag || power_flag) {
+				rv = poly_div(&equation[j], len2, &equation[i+1], len1, NULL);
+			} else {
+				rv = smart_div(&equation[j], len2, &equation[i+1], len1);
+			}
+			zero_remainder = (rv > 0 && REMAINDER_IS_ZERO());
+			if (power_flag && !zero_remainder) {
+				rv = 0;
+			}
+			if (rv > 0) { /* if successful and the result is smaller than the original: */
+				if ((n_tlhs + 2 + n_trhs + len1) > n_tokens)
+					error_huge();
+				for (k = 0; k < n_tlhs; k++)
+					tlhs[k].level++;
+				tlhs[n_tlhs].level = 1;
+				tlhs[n_tlhs].kind = OPERATOR;
+				tlhs[n_tlhs].token.operatr = PLUS;
+				n_tlhs++;
+				for (k = 0; k < n_trhs; k++)
+					trhs[k].level += 2;
+				blt(&tlhs[n_tlhs], trhs, n_trhs * sizeof(token_type));
+				n_tlhs += n_trhs;
+				tlhs[n_tlhs].level = 2;
+				tlhs[n_tlhs].kind = OPERATOR;
+				tlhs[n_tlhs].token.operatr = DIVIDE;
+				n_tlhs++;
+				k = n_tlhs;
+				blt(&tlhs[n_tlhs], &equation[i+1], len1 * sizeof(token_type));
+				n_tlhs += len1;
+				for (; k < n_tlhs; k++)
+					tlhs[k].level += 2;
+				side_debug(3, &equation[j], len2);
+				side_debug(3, &equation[i+1], len1);
+				simpb_side(tlhs, &n_tlhs, false, true, 3);	/* parameters should match what's used in simpa_side() */
+				side_debug(3, tlhs, n_tlhs);
 				if (power_flag) {
-					power_flag = false;
-					goto next_thingy;
+					k = (var_count(tlhs, n_tlhs) <= var_count(&equation[j], len2));
+				} else {
+					k = ((var_count(tlhs, n_tlhs) + (n_tlhs >= len1 + 1 + len2)) <= (var_count(&equation[j], len2) + var_count(&equation[i+1], len1)));
 				}
-				if (flag == code) {
-					flag = !flag;
-					goto next_thingy;
+				if (k) {
+					for (k = 0; k < n_tlhs; k++)
+						tlhs[k].level += level;
+					if (power_flag) {
+						if ((*np - len2 + n_tlhs + 2) > n_tokens)
+							error_huge();
+						for (k = i + 2 + len1; k <= i + real_len1; k++) {
+							equation[k].level++;
+						}
+						blt(&equation[i+real_len1+3], &equation[k], (*np - k) * sizeof(token_type));
+						*np += 2;
+						equation[k].level = level + 2;
+						equation[k].kind = OPERATOR;
+						equation[k].token.operatr = MINUS;
+						k++;
+						equation[k].level = level + 2;
+						equation[k].kind = CONSTANT;
+						equation[k].token.constant = 1.0;
+						if (i < j) {
+							j += 2;
+						}
+					} else {
+						if ((*np - (len1 + 1 + len2) + n_tlhs) > n_tokens)
+							error_huge();
+						blt(&equation[i], &equation[i+1+len1], (*np - (i + 1 + len1)) * sizeof(token_type));
+						*np -= len1 + 1;
+						if (i < j) {
+							j -= len1 + 1;
+						}
+					}
+					blt(&equation[j+n_tlhs], &equation[j+len2], (*np - (j + len2)) * sizeof(token_type));
+					*np -= len2 - n_tlhs;
+					blt(&equation[j], tlhs, n_tlhs * sizeof(token_type));
+					if (flag || power_flag) {
+						debug_string(1, _("Polynomial division successful."));
+					} else {
+						debug_string(1, _("Smart division successful."));
+					}
+					side_debug(3, equation, *np);
+					return true;
 				}
 			}
+			if (power_flag) {
+				power_flag = false;
+				goto next_thingy;
+			}
+			if (flag == code) {
+				flag = !flag;
+				goto next_thingy;
+			}
 		}
 	}
 	for (i = loc; i < *np && equation[i].level >= level;) {
@@ -1252,9 +1337,13 @@ next_thingy:
 }
 
 /*
- * Do polynomial division.
+ * Do generalized and accurate polynomial division.
+ * This is the main polynomial division function for Mathomatic.
+ * Works with any number of variables and any pair of expressions.
  *
- * Returns non-zero if successful, 2 if remainder is 0.
+ * Returns non-zero if successful: 2 if remainder is 0,
+ * 1 if result is smaller than the original two expressions,
+ * negative if result is larger.
  * Quotient is returned in tlhs[] and remainder in trhs[].
  *
  * If *vp is 0, automatically select the best base variable and return it in *vp.
@@ -1265,7 +1354,7 @@ token_type	*d1;		/* pointer to dividend */
 int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
-long		*vp;		/* variable pointer to base variable */
+long		*vp;		/* pointer to base variable */
 {
 	int		i;
 	int		rv;
@@ -1289,8 +1378,11 @@ long		*vp;		/* variable pointer to base variable */
 	return rv;
 }
 
+#define	MAX_GREATEST_POWER_TERMS	50	/* a limit to keep polynomial division from taking too long */
+
 /*
- * Do the actual polynomial division using a simple generalized polynomial long division algorithm.
+ * Do the actual polynomial division using a simple,
+ * generalized, polynomial long division algorithm.
  */
 static int
 poly_div_sub(d1, len1, d2, len2, vp)
@@ -1308,19 +1400,23 @@ long		*vp;
 	double		last_power, divisor_power, d;
 	int		sum_size;
 	int		last_count, count;	/* dividend number of terms raised to the highest power */
+	int		divisor_count;		/* divisor number of terms raised to the highest power */
+	long		tmp_v = 0;
 
+	if (vp == NULL)
+		vp = &tmp_v;
 	if (len1 > n_tokens || len2 > n_tokens)
 		return false;
 	/* Copy the source polynomials to where we want them (tlhs and trhs). */
 	if (trhs != d1) {
 		blt(trhs, d1, len1 * sizeof(token_type));
-		n_trhs = len1;
 	}
+	n_trhs = len1;
 	if (tlhs != d2) {
 		blt(tlhs, d2, len2 * sizeof(token_type));
-		n_tlhs = len2;
 	}
-	/* Do the basic unfactoring and simplification. */
+	n_tlhs = len2;
+	/* Do the basic unfactoring and simplification of the dividend and divisor. */
 	uf_simp(trhs, &n_trhs);
 	uf_simp(tlhs, &n_tlhs);
 	if (*vp == 0) {
@@ -1340,15 +1436,18 @@ long		*vp;
 	/* Determine divide_flag and if the polynomials can be divided. */
 	divide_flag = 2;
 	last_count = find_greatest_power(trhs, n_trhs, vp, &last_power, &t1, &len_t1, &divide_flag);
-	find_greatest_power(tlhs, n_tlhs, vp, &divisor_power, &t2, &len_t2, &divide_flag);
+	divisor_count = find_greatest_power(tlhs, n_tlhs, vp, &divisor_power, &t2, &len_t2, &divide_flag);
 	if (divisor_power <= 0 || last_power < divisor_power) {
 		divide_flag = !divide_flag;
 		last_count = find_greatest_power(trhs, n_trhs, vp, &last_power, &t1, &len_t1, &divide_flag);
-		find_greatest_power(tlhs, n_tlhs, vp, &divisor_power, &t2, &len_t2, &divide_flag);
+		divisor_count = find_greatest_power(tlhs, n_tlhs, vp, &divisor_power, &t2, &len_t2, &divide_flag);
 		if (divisor_power <= 0 || last_power < divisor_power) {
 			return false;
 		}
 	}
+	if (divisor_count > 1 || last_count > MAX_GREATEST_POWER_TERMS)	/* Need recursion and factoring to do more than this. */
+		return false;
+
 	/* Initialize the quotient. */
 	n_quotient = 1;
 	quotient[0] = zero_token;
@@ -1448,10 +1547,12 @@ long		*vp;
 				else
 					return -1;
 			}
-			return true;
+			return 1;
 		} else if (d < last_power) {
 			last_power = d;
 			last_count = count;
+			if (last_count > MAX_GREATEST_POWER_TERMS)	/* Need factoring to do more than this. */
+				return false;
 		} else if (d > last_power) {
 			return false;
 		} else {
@@ -1918,18 +2019,19 @@ int		n1, loc;
  * This routine finds the additive term raised the highest power,
  * with ordering and much flexibility.
  *
- * The number of terms raised to the highest power is returned.
+ * The number of terms raised to the highest power is returned, unless *vp1 == 0,
+ * then 0 is returned and *vp1 is set to a valid base variable, if any.
  */
 int
 find_greatest_power(p1, n1, vp1, pp1, tp1, lentp1, dcodep)
 token_type	*p1;		/* pointer to expression */
 int		n1;		/* length of expression */
-long		*vp1;		/* variable pointer */
-double		*pp1;
-int		*tp1, *lentp1;
-int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
-				/* for example: for x^5 it is false, for 1/x^5 it is true */
-				/* if equal to 3, ignore */
+long		*vp1;		/* base variable */
+double		*pp1;		/* returned power of the returned term, which will be the highest power */
+int		*tp1, *lentp1;	/* the returned term index and length */
+int		*dcodep;	/* divide flag pointer indicates if term is a denominator; */
+				/* for example: for x^5 it is false, for 1/x^5 it is true. */
+				/* If equal to 2, set it; if equal to 3, ignore it. */
 {
 	int		i, j, k, ii;
 	double		d;
@@ -1945,6 +2047,7 @@ int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 	CLEAR_ARRAY(last_va);
 	*pp1 = 0.0;
 	*tp1 = -1;
+	*lentp1 = 0;
 	rv = *dcodep;
 	for (j = 0, i = 1;; i += 2) {
 		if (i >= n1 || ((p1[i].token.operatr == PLUS || p1[i].token.operatr == MINUS) && p1[i].level == 1)) {
@@ -1979,7 +2082,7 @@ int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 								}
 								break;
 							}
-						} else {
+						} else if ((p1[k].token.variable & VAR_MASK) > SIGN) {
 							v = p1[k].token.variable;
 							*pp1 = 1.0;
 							*tp1 = j;
@@ -2054,7 +2157,7 @@ int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 							}
 							break;
 						}
-					} else {
+					} else if ((p1[k-1].token.variable & VAR_MASK) > SIGN) {
 						was_power = true;
 						if (d > *pp1) {
 							v = p1[k-1].token.variable;
@@ -2073,8 +2176,9 @@ int		*dcodep;	/* divide flag pointer indicates if term is a denominator */
 			}
 		}
 	}
-	if (*vp1 == 0)
+	if (*vp1 == 0) {
 		*vp1 = v;
+	}
 	if (*tp1 >= 0) {
 		for (i = *tp1 + 1; i < n1; i += 2) {
 			if ((p1[i].token.operatr == PLUS || p1[i].token.operatr == MINUS)
