@@ -2,6 +2,24 @@
  * Find and display the minimum sum of the squares for integers.
  *
  * Copyright (C) 2007 George Gesslein II.
+ 
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
+ 
  */
 
 /*
@@ -9,23 +27,28 @@
  *
  * This program finds the minimum number of positive integers squared
  * and summed to equal a given positive integer.  If the number of squared
- * integers is 2, all combinations with 2 squares are displayed,
+ * integers is 2 (default "multi"), all combinations with 2 squares are displayed,
  * otherwise only the first solution found is displayed.
  *
  * If nothing is specified on the command line, the program starts at 0 and
  * counts up.
  *
- * This file is mostly useful for testing the long integer square root function lsqrt().
+ * This file is mostly useful for testing the long integer square root function lsqrt() in file "lsqrt.c".
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <errno.h>
+#include <assert.h>
 
 #define	true	1
 #define	false	0
 
+#define	squared(a)	((a) * (a))
+
+int	multi = 2;	/* display all combinations for this number of squares or less */
+
 long	squares[4];
 
 long	lsqrt(long y);
@@ -50,10 +73,8 @@ try_again:
 			save = d2;
 		}
 		squares[i] = lsqrt(d2);
-		if (squares[i] < 0) {
-			exit(1);
-		}
-		d2 -= (squares[i] * squares[i]);
+		assert(squares[i] >= 0);
+		d2 -= squared(squares[i]);
 		i++;
 		if (i >= n) {
 			break;
@@ -61,11 +82,11 @@ try_again:
 	}
 	if (d2 == 0) {
 		for (i = 0; i < n; i++) {
-			d2 += (squares[i] * squares[i]);
+			d2 += squared(squares[i]);
 		}
 		if (d2 != d1) {
-			fprintf(stderr, "Result doesn't compare identical to original number!\n");
-			exit(1);
+			fprintf(stderr, "Error: Result doesn't compare identical to original number!\n");
+			exit(EXIT_FAILURE);
 		}
 		for (i = 0; i < (n - 1); i++) {
 			if (squares[i] < squares[i+1]) {
@@ -80,11 +101,8 @@ try_again:
 		}
 		printf("\n");
 skip_print:
-		if (!found_one) {
-			fprintf(stderr, "Found bug!\n");
-			exit(1);
-		}
-		if (n != 2) {
+		assert(found_one);
+		if (n < 2 || n > multi) {
 			return found_one;
 		}
 	}
@@ -92,21 +110,21 @@ skip_print:
 	case 4:
 		if (squares[2] > squares[n-1]) {
 			squares[2] -= 1;
-			d2 = save - (squares[2] * squares[2]);
+			d2 = save - squared(squares[2]);
 			i = 3;
 			goto try_again;
 		}
 	case 3:
 		if (squares[1] > squares[n-1]) {
 			squares[1] -= 1;
-			d2 = d1 - (squares[0] * squares[0]) - (squares[1] * squares[1]);
+			d2 = d1 - squared(squares[0]) - squared(squares[1]);
 			i = 2;
 			goto try_again;
 		}
 	case 2:
 		if (squares[0] > squares[n-1]) {
 			squares[0] -= 1;
-			d2 = d1 - (squares[0] * squares[0]);
+			d2 = d1 - squared(squares[0]);
 			i = 1;
 			goto try_again;
 		}
@@ -115,9 +133,9 @@ skip_print:
 }
 
 /*
- * Display the shortest sum of the squares for long integer "d1".
+ * Display the shortest sums of squares for long integer "d1".
  *
- * Return number of squares.
+ * Return the minimum number of summed squares required to represent "d1".
  */
 int
 findsq(long d1)
@@ -128,11 +146,10 @@ findsq(long d1)
 		return 2;
 	if (sumsq(d1, 3))
 		return 3;
-	if (!sumsq(d1, 4)) {
-		fprintf(stderr, "Whoops!  Can't find the sum of four squares that equal %ld.\n", d1);
-		exit(1);
-	}
-	return 4;
+	if (sumsq(d1, 4))
+		return 4;
+	fprintf(stderr, "Whoops!  Can't find the sum of four squares that equal %ld.\n", d1);
+	exit(EXIT_FAILURE);
 }
 
 int
@@ -146,9 +163,13 @@ main(int argc, char *argv[])
 		for (i = 1; i < argc; i++) {
 			errno = 0;
 			d1 = strtol(argv[i], &cp, 10);
-			if (errno || d1 <= 0) {
-				fprintf(stderr, "Invalid number.\n");
-				exit(1);
+			if (errno) {
+				perror(NULL);
+				exit(EXIT_FAILURE);
+			}
+			if (argv[i][0] == '\0' || *cp || d1 <= 0) {
+				fprintf(stderr, "Invalid number: \"%s\".\n", argv[i]);
+				exit(EXIT_FAILURE);
 			}
 			findsq(d1);
 		}
@@ -157,5 +178,5 @@ main(int argc, char *argv[])
 			findsq(d1);
 		}
 	}
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
