@@ -1,21 +1,24 @@
 /*
  * Mathomatic simplifying routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -30,10 +33,10 @@
  */
 #define	MAX_COMPARE_TERMS	(DEFAULT_N_TOKENS / 6)
 
-static int	org_recurse();
-static int	const_recurse();
-static int	compare_recurse();
-static int	order_recurse();
+static int org_recurse(token_type *equation, int *np, int loc, int level, int *elocp);
+static int const_recurse(token_type *equation, int *np, int loc, int level, int iflag);
+static int compare_recurse(token_type *p1, int n1, int l1, token_type *p2, int n2, int l2, int *diff_signp);
+static int order_recurse(token_type *equation, int *np, int loc, int level);
 
 /*
  * Fix up levels of parentheses in an equation side.
@@ -172,7 +175,18 @@ int		*np;		/* pointer to length of equation side */
 	if (abort_flag) {
 		/* Control-C pressed, gracefully return to main prompt and leave unsimplified */
 		abort_flag = false;
+#if	DEBUG && !SILENT
+		char	*cp, buf[100];
+
+		my_strlcpy(prompt_str, _("Enter debug level or empty line to abort: "), sizeof(prompt_str));
+		if ((cp = get_string(buf, sizeof(buf))) == NULL || *cp == '\0') {
+			longjmp(jmp_save, 13);
+		} else {
+			debug_level = decstrtol(cp, NULL);
+		}
+#else
 		longjmp(jmp_save, 13);
+#endif
 	}
 	side_debug(6, equation, *np);
 	do {
@@ -233,7 +247,7 @@ int	n;	/* equation space number to simplify */
 	if (n_lhs[n] <= 0)
 		return;
 	simp_loop(lhs[n], &n_lhs[n]);
-	if (n_rhs[n]) {
+	if (n_rhs[n] > 0) {
 		simp_loop(rhs[n], &n_rhs[n]);
 	}
 }
@@ -243,13 +257,29 @@ int	n;	/* equation space number to simplify */
  * Trivial factoring is done.
  */
 void
-simp_side(equation, np)
+mid_simp_side(equation, np)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of equation side */
 {
 	simp_ssub(equation, np, 0L, 1.0, true, true, 6);
 }
 
+/*
+ * For quick, mid-range simplification of an equation space.
+ * Trivial factoring is done.
+ */
+void
+mid_simp_equation(n)
+int	n;	/* equation space number to simplify */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	mid_simp_side(lhs[n], &n_lhs[n]);
+	if (n_rhs[n] > 0) {
+		mid_simp_side(rhs[n], &n_rhs[n]);
+	}
+}
+
 /*
  * This function is the mid-range simplifier used by the solver.
  */
@@ -275,24 +305,54 @@ token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to length of equation side */
 long		v;		/* variable to factor, 0 for all variables */
 {
-	if (*np == 0)
-		return;
 	simp_ssub(equation, np, v, 0.0, v == 0, true, 6);
 }
 
 /*
- * Factor only the specified variable "v".
- * If v is IMAGINARY, do complex number simplification.
+ * For factoring an equation space like the factor command.
+ * Trivial factoring is done.
+ */
+void
+simpv_equation(n, v)
+int	n;	/* equation space number to simplify */
+long	v;	/* Mathomatic variable to factor or 0 */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	simpv_side(lhs[n], &n_lhs[n], v);
+	if (n_rhs[n] > 0) {
+		simpv_side(rhs[n], &n_rhs[n], v);
+	}
+}
+
+/*
+ * Factor out and simplify imaginary constants in an equation side.
+ * Do complex number root approximation and other complex number simplification.
+ *
+ * Return true if anything was approximated.
+ */
+int
+factor_imaginary(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side */
+int		*np;		/* pointer to equation side length */
+{
+	int	rv;
+
+	rv = approximate_complex_roots(equation, np);
+	factorv(equation, np, IMAGINARY);
+	return rv;
+}
+
+/*
+ * Factor out only the specified variable "v" and simplify a little.
+ * Does not do any approximation.
  */
 void
 factorv(equation, np, v)
 token_type	*equation;	/* pointer to the beginning of equation side to factor */
 int		*np;		/* pointer to equation side length */
-long		v;		/* variable to factor */
+long		v;		/* Mathomatic variable to factor */
 {
-	if (v == IMAGINARY) {
-		approximate_complex_roots(equation, np);
-	}
 	do {
 		do {
 			simp_loop(equation, np);
@@ -311,19 +371,33 @@ int		*np;
 {
 	approximate_roots = true;
 	subst_constants(equation, np);
-	simp_side(equation, np);
-	uf_power(equation, np);
-	factorv(equation, np, IMAGINARY);
+	mid_simp_side(equation, np);
+	factor_imaginary(equation, np);
 	ufactor(equation, np);
-	factorv(equation, np, IMAGINARY);
+	factor_imaginary(equation, np);
 	uf_simp(equation, np);
-	factorv(equation, np, IMAGINARY);
-	simp_side(equation, np);
-	make_fractions(equation, np);
+	factor_imaginary(equation, np);
+	mid_simp_side(equation, np);
+	make_simple_fractions(equation, np);
 	uf_tsimp(equation, np);
 	approximate_roots = false;
 }
 
+/*
+ * Approximate an equation side for the approximate command.
+ */
+void
+approximate(equation, np)
+token_type	*equation;
+int		*np;
+{
+	subst_constants(equation, np);
+	approximate_roots = true;
+	simp_loop(equation, np);
+	factor_imaginary(equation, np);
+	approximate_roots = false;
+}
+
 /*
  * Try to eliminate imaginary numbers (i#) from an equation side by converting
  * expressions like (i#*(b^.5)) to ((-1*b)^.5).
@@ -389,6 +463,8 @@ int		*np;
 /*
  * Combine all like denominators containing variable "v".
  * Don't call factor_times().
+ *
+ * For beauty simplifier simpb_side() below.
  */
 void
 simp2_divide(equation, np, v, fc_level)
@@ -411,7 +487,7 @@ int		fc_level;
 }
 
 /*
- * Compare function for qsort(3) within simpb_side().
+ * Compare function for qsort(3) within simpb_side() below.
  */
 static int
 simpb_vcmp(p1, p2)
@@ -453,7 +529,11 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 	long		v1, last_v;	/* Mathomatic variables */
 	sort_type	va[MAX_VARS];	/* array of all real variables found in equation side */
 
+#if	1
+	elim_loop(equation, np);
+#else
 	simp_loop(equation, np);
+#endif
 	if (uf_power_flag) {
 		uf_allpower(equation, np);
 	}
@@ -502,6 +582,13 @@ int		fc_level;	/* factor constants code, passed to factor_constants() */
 		}
 	}
 	simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
+#if	0	/* Seems to complicate and change constants slightly when enabled. */
+	if (power_flag) {
+		make_simple_fractions(equation, np);
+		factor_power(equation, np);
+		simp_ssub(equation, np, MATCH_ANY, 0.0, power_flag, true, fc_level);
+	}
+#endif
 }
 
 /*
@@ -517,20 +604,21 @@ int		*np;		/* pointer to length of equation side */
 {
 	if (*np <= 0)
 		return;
-
+	simp_loop(equation, np);
+	poly_factor(equation, np, true);
 	do {
 		do {
 			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 5);
+				simp_ssub(equation, np, 0L, 0.0, false, true, 5);
 			} while (poly_gcd_simp(equation, np));
 		} while (uf_power(equation, np));
 	} while (super_factor(equation, np, 2));
 	side_debug(2, equation, *np);
 
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	uf_tsimp(equation, np);
 #if	0	/* Not really needed, but makes it end up the same as the simplify command. */
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
 	simpb_side(equation, np, true, true, 3);
@@ -552,7 +640,7 @@ void
 simpa_side(equation, np, quick_flag, frac_flag)
 token_type	*equation;	/* pointer to the beginning of equation side to simplify */
 int		*np;		/* pointer to length of the equation side */
-int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int		quick_flag;	/* "simplify quick" option, simpler with no (x+1)^2 expansion */
 int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
 {
 	int		i;
@@ -561,14 +649,15 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 
 	if (*np <= 1)	/* no need to simplify a single constant or variable */
 		return;
+	debug_string(2, "Simplify input:");
 	side_debug(2, equation, *np);
-	simpb_side(equation, np, false, true, 1);
+	simp_loop(equation, np);
+	simpb_side(equation, np, false, true, 5);
 	if (rationalize_denominators) {
 		rationalize(equation, np);
 	}
 	unsimp_power(equation, np);
-	uf_times(equation, np);
-	simp_loop(equation, np);
+	uf_tsimp(equation, np);
 
 /* Here is the only place in Mathomatic that we do complete modulus (%) simplification: */
 	uf_pplus(equation, np);
@@ -581,33 +670,38 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	simp_i(equation, np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, true, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, true, 5);
 	unsimp_power(equation, np);
 	uf_neg_help(equation, np);
-	uf_times(equation, np);
+	uf_tsimp(equation, np);
 	do {
 		do {
-			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 2);
-			} while (poly_gcd_simp(equation, np));
+			simp_ssub(equation, np, 0L, 1.0, false, true, 6);
 		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
+	} while (!quick_flag && super_factor(equation, np, 2));
+	if (poly_gcd_simp(equation, np)) {
+		simp_ssub(equation, np, 0L, 1.0, false, true, 6);
+	}
+	side_debug(2, equation, *np);
 	unsimp_power(equation, np);
 	uf_times(equation, np);
 	factorv(equation, np, IMAGINARY);
 	uf_pplus(equation, np);
-	simp_ssub(equation, np, 0L, 1.0, true, false, 1);
+	simp_ssub(equation, np, 0L, 1.0, true, false, 5);
+	if (poly_gcd_simp(equation, np)) {
+		factorv(equation, np, IMAGINARY);
+		uf_pplus(equation, np);
+		simp_ssub(equation, np, 0L, 1.0, true, false, 5);
+	}
 	uf_times(equation, np);
 	uf_pplus(equation, np);
-	factorv(equation, np, IMAGINARY);
+	factor_imaginary(equation, np);
 	uf_power(equation, np);
 	do {
 		do {
-			do {
-				simp_ssub(equation, np, 0L, 1.0, false, true, 6);
-			} while (poly_gcd_simp(equation, np));
+			simp_ssub(equation, np, 0L, 1.0, false, true, 6);
 		} while (uf_power(equation, np));
-	} while (super_factor(equation, np, 2));
+	} while (!quick_flag && super_factor(equation, np, 2));
 
 /* Here we do the greatest expansion; if it fails, do less expansion. */
 	partial_flag = frac_flag;
@@ -666,14 +760,15 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 		}
 		break;
 	}
-/*	simpb_side(equation, np, true, true, 1); */
+	debug_string(2, "Raw simplify result after applying polynomial operations:");
+	side_debug(2, equation, *np);
 	simp_constant_power(equation, np);
 	simp_ssub(equation, np, 0L, 1.0, true, true, 5);
 	unsimp_power(equation, np);
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	factor_power(equation, np);
 	uf_tsimp(equation, np);
-	make_fractions(equation, np);
+	make_simple_fractions(equation, np);
 	uf_power(equation, np);
 	integer_root_simp(equation, np);
 	simpb_side(equation, np, true, true, 3);
@@ -696,6 +791,8 @@ int		*np;		/* pointer to length of the equation side */
 int		quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
 int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
 {
+	if (*np <= 1)	/* no need to simplify a single constant or variable */
+		return;
 	simpa_side(equation, np, quick_flag, frac_flag);
 	if (repeat_flag) {
 		do {
@@ -710,6 +807,34 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 }
 
+/*
+ * This routine is used by the fraction command.
+ * It repeats the simplification until the smallest expression is achieved,
+ * if repeat_flag is true.
+ *
+ * Globals tes[], tlhs[], and trhs[] are wiped out.
+ */
+void
+simple_frac_repeat_side(equation, np)
+token_type	*equation;	/* pointer to the beginning of equation side to simplify */
+int		*np;		/* pointer to length of the equation side */
+{
+	if (*np <= 0)
+		return;
+	simple_frac_side(equation, np);
+	if (repeat_flag) {
+		do {
+			n_tes = *np;
+			blt(tes, equation, n_tes * sizeof(token_type));
+			simple_frac_side(equation, np);
+		} while (*np < n_tes);
+		if (*np != n_tes) {
+			*np = n_tes;
+			blt(equation, tes, n_tes * sizeof(token_type));
+		}
+	}
+}
+
 /*
  * Commonly used quick simplify routine that doesn't factor.
  *
@@ -740,13 +865,14 @@ int		*np;		/* pointer to length of equation side */
 
 /*
  * Convert (x^n)^m to x^(n*m) when appropriate.
- * Recently fixed to work the same as Maxima when symb_flag is false,
- * otherwise always converts (x^n)^m to x^(n*m),
- * which sometimes simplifies more.
+ * Fixed to work the same as Maxima when symb_flag is false,
+ * otherwise always converts (x^n)^m to x^(n*m), which sometimes simplifies more.
+ *
  * Earlier versions of Mathomatic always converted (x^n)^m to x^(n*m),
  * which is the wrong thing to do, because they are not always equivalent.
+ * That is now the job of the "simplify symbolic" command, which sets symb_flag to true.
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 simp_pp(equation, np)
@@ -759,15 +885,19 @@ int		*np;
 	double		numerator, denominator;
 
 	for (i = 1; i < *np; i += 2) {
+#if	DEBUG
+		if (equation[i].kind != OPERATOR) {
+			error_bug("Bug found in simp_pp(), operators are misplaced.");
+		}
+#endif
 		if (equation[i].token.operatr != POWER)
 			continue;
 		ilevel = equation[i].level;
 		for (j = i + 2; j < *np; j += 2) {
 			jlevel = equation[j].level;
 			if (jlevel == ilevel - 1 && equation[j].token.operatr == POWER) {
-				if (!symb_flag) {
-					if (jlevel == equation[j+1].level
-					    && equation[j+1].kind == CONSTANT) {
+				if (!symb_flag && (equation[i-1].level != ilevel || equation[i-1].kind != CONSTANT || equation[i-1].token.constant < 0)) {
+					if (jlevel == equation[j+1].level && equation[j+1].kind == CONSTANT) {
 						f_to_fraction(equation[j+1].token.constant, &numerator, &denominator);
 						if (fmod(denominator, 2.0) == 0.0) {
 							if ((i + 2) == j && equation[i+1].kind == CONSTANT) {
@@ -806,9 +936,9 @@ int		*np;
 }
 
 /*
- * Simplify things like 12^(1/2) to 2*3^(1/2).
+ * Simplify surds like 12^(1/2) to 2*3^(1/2).
  *
- * Return true if expression was modified.
+ * Return true if equation side was modified.
  */
 int
 integer_root_simp(equation, np)
@@ -818,41 +948,44 @@ int		*np;
 	int	modified = false;
 	int	i, j, k;
 	int	level;
-	double	d1, d2;
-	int	root;
+	double	d1, d2, numerator, denominator;
 
-	for (i = 1; i < (*np - 2); i += 2) {
+	for (i = 1; (i + 3) < *np; i += 2) {
+#if	DEBUG
+		if (equation[i].kind != OPERATOR) {
+			error_bug("Bug found in integer_root_simp(), operators are misplaced.");
+		}
+#endif
 		if (equation[i].token.operatr == POWER) {
 			level = equation[i].level;
 			if (equation[i-1].level == level
+			    && equation[i+1].level == level + 1
 			    && equation[i+2].level == level + 1
-			    && equation[i+2].token.operatr == DIVIDE
 			    && equation[i+3].level == level + 1
+			    && equation[i+2].token.operatr == DIVIDE
 			    && equation[i-1].kind == CONSTANT
 			    && equation[i+1].kind == CONSTANT
 			    && equation[i+3].kind == CONSTANT) {
-				if (i + 4 < *np && equation[i+4].level >= level)
+				if ((i + 4) < *np && equation[i+4].level >= level)
 					continue;
-				d2 = equation[i+1].token.constant;
-				if (d2 < 1.0 || d2 > 50.0 || fmod(d2, 1.0) != 0.0)
+				numerator = equation[i+1].token.constant;
+				if (numerator > 50.0 || numerator < 1.0 || fmod(numerator, 1.0) != 0.0)
 					continue;
 				errno = 0;
-				d2 = pow(equation[i-1].token.constant, d2);
+				d2 = pow(equation[i-1].token.constant, numerator);
 				if (errno) {
 					continue;
 				}
-				if (equation[i+3].token.constant > 50.0 || equation[i+3].token.constant < 2.0)
-					continue;
-				root = equation[i+3].token.constant;
-				if (root != equation[i+3].token.constant || root < 2)
+				denominator = equation[i+3].token.constant;
+				if (denominator > 50.0 || denominator < 2.0 || fmod(denominator, 1.0) != 0.0)
 					continue;
 				if (!factor_one(d2))
 					continue;
 				d1 = 1.0;
 				for (j = 0; j < uno; j++) {
-					while (ucnt[j] >= root) {
+					while (ucnt[j] >= denominator) {
 						d1 *= unique[j];
-						ucnt[j] -= root;
+						ucnt[j] -= denominator;
 					}
 				}
 				if (d1 == 1.0)
@@ -862,67 +995,25 @@ int		*np;
 				}
 				equation[i+1].token.constant = 1.0;
 				equation[i-1].token.constant = multiply_out_unique();
-				k = i - 1;
-				j = i + 4;
-				for (; k < j; k++) {
+				for (k = i - 1; k < i + 4; k++) {
 					equation[k].level++;
 				}
 				blt(&equation[i+1], &equation[i-1], (*np - (i - 1)) * sizeof(token_type));
 				*np += 2;
-				equation[i].level = level;
-				equation[i].kind = OPERATOR;
-				equation[i].token.operatr = TIMES;
 				equation[i-1].level = level;
 				equation[i-1].kind = CONSTANT;
 				equation[i-1].token.constant = d1;
+				equation[i].level = level;
+				equation[i].kind = OPERATOR;
+				equation[i].token.operatr = TIMES;
 				modified = true;
-				i += 2;
+				i += 4;
 			}
 		}
 	}
 	return modified;
 }
 
-static int
-sp2_sub(equation, np, i)
-token_type	*equation;
-int		*np, i;
-{
-	int	j;
-	int	level;
-
-	level = equation[i].level;
-	if (equation[i-1].level != level || equation[i-1].kind != CONSTANT)
-		return false;
-	if (equation[i+1].level != level + 1 || equation[i+1].kind != CONSTANT
-	    || equation[i+1].token.constant == 1.0)
-		return false;
-	j = i + 2;
-	if (j >= *np || equation[j].level != level + 1)
-		return false;
-	switch (equation[j].token.operatr) {
-	case TIMES:
-		break;
-	case DIVIDE:
-		if (*np + 2 > n_tokens) {
-			error_huge();
-		}
-		blt(&equation[j+2], &equation[j], (*np - j) * sizeof(token_type));
-		*np += 2;
-		equation[j+1].level = level + 1;
-		equation[j+1].kind = CONSTANT;
-		equation[j+1].token.constant = 1.0;
-		break;
-	default:
-		return false;
-	}
-	equation[j].level = level;
-	equation[j].token.operatr = POWER;
-	equation[i-1].level++;
-	equation[i].level++;
-	return true;
-}
-
 /*
  * Simplify constant^(constant*x) to (constant^constant)^x.
  * I am not sure if this is a good thing, because it changes the base of the exponent.
@@ -935,73 +1026,50 @@ simp_constant_power(equation, np)
 token_type	*equation;
 int		*np;
 {
+	int	i, j;
+	int	level;
 	int	modified = false;
-	int	i;
 
-	if (symb_flag) {
-		for (i = 1; i < *np; i += 2) {
-			if (equation[i].token.operatr == POWER) {
-				modified |= sp2_sub(equation, np, i);
-			}
+	for (i = 1; i < *np; i += 2) {
+		if (equation[i].token.operatr != POWER) {
+			continue;
 		}
+		level = equation[i].level;
+		if (equation[i-1].level != level || equation[i-1].kind != CONSTANT)
+			continue;
+		if (equation[i-1].token.constant < 0 && !symb_flag)
+			continue;
+		if (equation[i+1].level != level + 1 || equation[i+1].kind != CONSTANT
+		    || equation[i+1].token.constant == 1.0)
+			continue;
+		j = i + 2;
+		if (j >= *np || equation[j].level != level + 1)
+			continue;
+		switch (equation[j].token.operatr) {
+		case TIMES:
+			break;
+		case DIVIDE:
+			if (*np + 2 > n_tokens) {
+				error_huge();
+			}
+			blt(&equation[j+2], &equation[j], (*np - j) * sizeof(token_type));
+			*np += 2;
+			equation[j+1].level = level + 1;
+			equation[j+1].kind = CONSTANT;
+			equation[j+1].token.constant = 1.0;
+			break;
+		default:
+			continue;
+		}
+		equation[j].level = level;
+		equation[j].token.operatr = POWER;
+		equation[i-1].level++;
+		equation[i].level++;
+		modified = true;
 	}
 	return modified;
 }
 
-static int
-sp_sub(equation, np, i)
-token_type	*equation;
-int		*np, i;
-{
-	int	j, k;
-	int	level;
-	int	op;
-
-	level = equation[i].level;
-	op = 0;
-	k = -1;
-	for (j = i + 1; j < *np && equation[j].level >= level; j++) {
-		if (equation[j].level == level + 1) {
-			if (equation[j].kind == OPERATOR) {
-				op = equation[j].token.operatr;
-			} else if (equation[j].kind == CONSTANT && equation[j].token.constant < 0.0) {
-				k = j;
-			}
-		}
-	}
-	if (j - i <= 2 && equation[i+1].kind == CONSTANT && equation[i+1].token.constant < 0.0) {
-		k = i + 1;
-	} else if (k < 0)
-		return false;
-	switch (op) {
-	case 0:
-	case TIMES:
-	case DIVIDE:
-		if (*np + 2 > n_tokens) {
-			error_huge();
-		}
-		equation[k].token.constant = -equation[k].token.constant;
-		for (k = i - 2;; k--) {
-			if (k < 0 || equation[k].level < level)
-				break;
-		}
-		k++;
-		for (i = k; i < j; i++)
-			equation[i].level++;
-		blt(&equation[k+2], &equation[k], (*np - k) * sizeof(token_type));
-		*np += 2;
-		equation[k].level = level;
-		equation[k].kind = CONSTANT;
-		equation[k].token.constant = 1.0;
-		k++;
-		equation[k].level = level;
-		equation[k].kind = OPERATOR;
-		equation[k].token.operatr = DIVIDE;
-		return true;
-	}
-	return false;
-}
-
 /*
  * Convert x^-y to 1/(x^y).
  *
@@ -1012,17 +1080,80 @@ simp2_power(equation, np)
 token_type	*equation;
 int		*np;
 {
+	int	i, i1, j, k;
+	int	level;
+	int	op;
 	int	modified = false;
-	int	i;
 
 	for (i = 1; i < *np; i += 2) {
 		if (equation[i].token.operatr == POWER) {
-			modified |= sp_sub(equation, np, i);
+			level = equation[i].level;
+			op = 0;
+			k = -1;
+			for (j = i + 1; j < *np && equation[j].level >= level; j++) {
+				if (equation[j].level == level + 1) {
+					if (equation[j].kind == OPERATOR) {
+						op = equation[j].token.operatr;
+					} else if (equation[j].kind == CONSTANT && equation[j].token.constant < 0.0) {
+						k = j;
+					}
+				}
+			}
+			if (j - i <= 2 && equation[i+1].kind == CONSTANT && equation[i+1].token.constant < 0.0) {
+				k = i + 1;
+			} else if (k < 0)
+				continue;
+			switch (op) {
+			case 0:
+			case TIMES:
+			case DIVIDE:
+				if (*np + 2 > n_tokens) {
+					error_huge();
+				}
+				equation[k].token.constant = -equation[k].token.constant;
+				for (k = i - 2;; k--) {
+					if (k < 0 || equation[k].level < level)
+						break;
+				}
+				k++;
+				for (i1 = k; i1 < j; i1++)
+					equation[i1].level++;
+				blt(&equation[k+2], &equation[k], (*np - k) * sizeof(token_type));
+				*np += 2;
+				equation[k].level = level;
+				equation[k].kind = CONSTANT;
+				equation[k].token.constant = 1.0;
+				k++;
+				equation[k].level = level;
+				equation[k].kind = OPERATOR;
+				equation[k].token.operatr = DIVIDE;
+				modified = true;
+			}
 		}
 	}
 	return modified;
 }
 
+/*
+ * fmod(3) in the standard C math library has some problems.
+ * Hopefully this fixes them.
+ */
+double
+fixed_fmod(k1, k2)
+double	k1, k2;
+{
+	double	d;
+
+	if (k2 == 0 || !isfinite(k1) || !isfinite(k2) || (fmod(k1, 1.0) == 0 && fmod(k2, 1.0) == 0)) {
+		k1 = fmod(k1, k2);
+	} else {
+		/* fmod() doesn't work well with fractional amounts. */
+		/* Another way to get the remainder of division: */
+		k1 = modf(k1 / k2, &d) * k2;
+	}
+	return k1;
+}
+
 /*
  * Combine two or more constants on the same level of parentheses.
  * If "iflag" is false, don't produce imaginary numbers.
@@ -1051,6 +1182,7 @@ double	*k1p;	/* Pointer to operand 1 and where to store the result of the calcul
 int	op2;	/* Operator 2; always exists and comes immediately before operand 2, and usually after operand 1. */
 double	k2;	/* Operand 2; ignored for unary operators. */
 {
+	int	sign = 1;
 	int	op1;
 	double	d, d1, d2;
 
@@ -1111,15 +1243,9 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		break;
 	case MODULUS:
 		if (k2 == 0) {
-			warning(_("Modulo 0 encountered, might be considered undefined."));
-		}
-		if (fmod(*k1p, 1.0) == 0 && fmod(k2, 1.0) == 0) {
-			*k1p = fmod(*k1p, k2);
-		} else {
-			/* fmod() doesn't work well with fractional amounts. */
-			/* Another way to get the remainder of division: */
-			*k1p = modf(*k1p / k2, &d) * k2;
+			warning(_("Modulo 0 encountered."));
 		}
+		*k1p = fixed_fmod(*k1p, k2);
 		if (modulus_mode && *k1p < 0.0) {
 			*k1p += fabs(k2);	/* make positive */
 		}
@@ -1164,20 +1290,8 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 			*k1p = d;
 		break;
 	case FACTORIAL:
-#if	_XOPEN_SOURCE >= 600 || _ISOC99_SOURCE || __USE_ISOC99
-		/* Use true gamma for factorial if available, it is the nicest gamma() function. */
-		d = tgamma(*k1p + 1.0);
-		if (errno) {	/* don't evaluate if overflow */
-			return false;
-		}
-#else
-#if	1	/* Set this to 0 if lgamma() (logarithm of gamma) doesn't exist. */
-		/* Not threadsafe, but neither is most of Mathomatic with its global static data areas. */
-		d = exp(lgamma(*k1p + 1.0)) * signgam;
-		if (errno) {	/* don't evaluate if overflow */
-			return false;
-		}
-#else
+#if	NOGAMMA
+#warning "Using integer factorial only."
 		/* calculate integer factorial if gamma() functions don't exist */
 		if (*k1p > 170.0 || *k1p < 0.0 || fmod(*k1p, 1.0) != 0.0) {
 			return false;
@@ -1186,6 +1300,25 @@ double	k2;	/* Operand 2; ignored for unary operators. */
 		for (d1 = 2.0; d1 <= *k1p; d1 += 1.0) {
 			d *= d1;
 		}
+#else
+#if	_XOPEN_SOURCE >= 600 || _ISOC99_SOURCE || __USE_ISOC99 || MINGW
+		/* Use true gamma for factorial if available, it is the nicest gamma() function. */
+		/* Problem is I don't know how to properly tell if it is available. */
+		d = tgamma(*k1p + 1.0);
+		if (errno) {	/* don't evaluate if overflow */
+			return false;
+		}
+#else
+#warning "Using lgamma_r(3); Not a problem, but tgamma(3) is more direct, if available."
+#if	0
+		d = exp(lgamma(*k1p + 1.0)) * signgam;
+#else	/* use reentrant version: */
+		d = exp(lgamma_r(*k1p + 1.0, &sign));
+		d *= sign;
+#endif
+		if (errno) {	/* don't evaluate if overflow */
+			return false;
+		}
 #endif
 #endif
 		*k1p = d;
@@ -1271,11 +1404,34 @@ beginning:
 					d3 = pow(-d1, d2);
 					check_err();
 					if (!always_positive(denominator)) {
-						if (!always_positive(numerator)) {
-							d3 = -d3;
+						if (*np + 2 > n_tokens) {
+							error_huge();
 						}
-						d1 = d3;
-						goto not_imaginary;
+						blt(&equation[loc1+2], &equation[loc1], (*np - loc1) * sizeof(token_type));
+						*np += 2;
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = CONSTANT;
+						equation[loc1].token.constant = -d1;
+						loc1++;
+						equation[loc1].level = level + 1;
+						equation[loc1].kind = OPERATOR;
+						equation[loc1].token.operatr = POWER;
+						equation[loc].level = level + 1;
+						equation[loc].kind = CONSTANT;
+						equation[loc].token.constant = d2;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = OPERATOR;
+						equation[loc].token.operatr = TIMES;
+						loc++;
+						equation[loc].level = level;
+						equation[loc].kind = CONSTANT;
+						if (always_positive(numerator)) {
+							equation[loc].token.constant = 1.0;
+						} else {
+							equation[loc].token.constant = -1.0;
+						}
+						return true;
 					}
 					if (!iflag)
 						return modified;
@@ -1326,7 +1482,6 @@ beginning:
 					}
 					return true;
 				} else {
-not_imaginary:
 					equation[loc1].token.constant = d1;
 					modified = true;
 					domain_check = false;
@@ -1625,11 +1780,13 @@ int		*diff_signp;	/* different sign flag pointer */
 {
 	int	l1, l2;
 
+#if	1
 	if (((n1 > n2) ? ((n1 + 1) / (n2 + 1)) : ((n2 + 1) / (n1 + 1))) > 3) {
 		/* expressions are grossly different in size, no need to compare, they are different */
 		*diff_signp = false;
 		return false;
 	}
+#endif
 	/* Find the proper ground levels of parentheses for the two sub-expressions: */
 	l1 = min_level(p1, n1);
 	l2 = min_level(p2, n2);
@@ -1638,6 +1795,8 @@ int		*diff_signp;	/* different sign flag pointer */
 
 /*
  * Recursively compare each parenthesized sub-expression.
+ * This is the most used function in Mathomatic.
+ * Optimizing or streamlining this would greatly speed things up.
  */
 static int
 compare_recurse(p1, n1, l1, p2, n2, l2, diff_signp)
@@ -1696,8 +1855,8 @@ int		n2, l2, *diff_signp;
 		}
 		return false;
 	}
-#if	0
-	if ((n1 & 1) != 1 || (n2 & 1) != 1) {
+#if	DEBUG
+	if (n1 < 1 || n2 < 1 || (n1 & 1) != 1 || (n2 & 1) != 1) {
 		error_bug("Programming error in call to compare_recurse().");
 	}
 #endif
@@ -2290,6 +2449,11 @@ int		*np;
 be_thorough:
 		if (i >= *np)
 			break;
+#if	DEBUG
+		if (equation[i].kind != OPERATOR) {
+			error_bug("Bug in rationalize().");
+		}
+#endif
 		if (equation[i].token.operatr == DIVIDE) {
 			div_level = equation[i].level;
 			count = 0;
@@ -2380,8 +2544,10 @@ do_again:
 						equation[k1].token.operatr = DIVIDE;
 						k1++;
 						blt(&equation[k1], scratch, k * sizeof(token_type));
-						i = k1 + k;
-						debug_string(1, "Square roots in denominator rationalized.");
+						k1 += k;
+						debug_string(1, "Square roots in denominator rationalized:");
+						side_debug(1, &equation[i+1], k1 - (i + 1));
+						i = k1;
 						modified = true;
 						goto be_thorough;
 					}
