@@ -1,21 +1,24 @@
 /*
  * Mathomatic symbolic solve routines.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -23,10 +26,10 @@
 
 #define	MAX_RAISE_POWER	20	/* Maximum number of times to increase power in solve function. */
 
-static int	flip();
-static int	g_of_f();
-static int	poly_solve();
-static int	increase();
+static int increase(double d, long v);
+static int poly_solve(long v);
+static int g_of_f(int op, token_type *operandp, token_type *side1p, int *side1np, token_type *side2p, int *side2np);
+static int flip(token_type *side1p, int *side1np, token_type *side2p, int *side2np);
 
 static int	repeat_count;
 static int	prev_n1, prev_n2;
@@ -69,15 +72,10 @@ int	have;	/* equation number to solve */
 			rv = solve_sub(rhs[want], n_rhs[want], rhs[have], &n_rhs[have], lhs[have], &n_lhs[have]);
 		}
 	}
-	/* Free the "want" equation space: */
-	n_lhs[want] = 0;
-	n_rhs[want] = 0;
 	blt(jmp_save, save_save, sizeof(jmp_save));
-#if	!SILENT
 	if (rv <= 0) {
-		printf(_("Solve failed.\n"));
+		printf(_("Solve failed for equation space #%d.\n"), have + 1);
 	}
-#endif
 	return(rv > 0);
 }
 
@@ -127,7 +125,11 @@ int		*rightnp;	/* pointer to length of RHS */
 	prev_n1 = 0;
 	prev_n2 = 0;
 	if (*leftnp <= 0 || *rightnp <= 0) {
+#if	LIBRARY || !HELP
+		error(_("Solving requires an equation."));
+#else
 		error(_("Please enter an equation or a command like \"help\"."));
+#endif
 		return false;
 	}
 	if (wantn != 1) {
@@ -148,12 +150,17 @@ int		*rightnp;	/* pointer to length of RHS */
 			n_trhs = *rightnp;
 			blt(trhs, rightp, n_trhs * sizeof(*rightp));
 			uf_simp(tlhs, &n_tlhs);
-			if (increase(1 / wantp[2].token.constant, v) != true) {
+			if ((i = increase(1 / wantp[2].token.constant, v)) != true) {
 				error(_("Unable to isolate root."));
+#if	!SILENT
+				if (i == 2) {
+					printf(_("No matching root found.\n"));
+				}
+#endif
 				return false;
 			}
 			list_tdebug(2);
-			simp_side(tlhs, &n_tlhs);
+			mid_simp_side(tlhs, &n_tlhs);
 			simp_loop(trhs, &n_trhs);
 			uf_simp(trhs, &n_trhs);
 			list_tdebug(1);
@@ -177,7 +184,7 @@ int		*rightnp;	/* pointer to length of RHS */
 		v = wantp->token.variable;
 		if (!found_var(trhs, n_trhs, v) && !found_var(tlhs, n_tlhs, v)) {
 			/* variable v is 0 or not found */
-			error(_("Variable not found."));
+			error(_("Solve variable not found."));
 			return false;
 		}
 		zsolve = false;
@@ -311,8 +318,7 @@ zero_simp:
 						if (p1 >= ep || (p1->kind == OPERATOR && p1->level == 1)) {
 							blt(b1 + 1, p1, (char *) ep - (char *) p1);
 							n_trhs -= p1 - (b1 + 1);
-							b1->kind = CONSTANT;
-							b1->token.constant = 1.0;
+							*b1 = one_token;
 							goto zero_simp;
 						}
 						if (p1->kind != CONSTANT && p1->kind != OPERATOR
@@ -427,9 +433,7 @@ zero_simp:
 					calc_simp(trhs, &n_trhs);
 					if (se_compare(tlhs, n_tlhs, trhs, n_trhs, &diff_sign) && !diff_sign) {
 						error(_("This equation is an identity."));
-#if	!SILENT
-						printf(_("That is, the LHS is identical to the RHS.\n"));
-#endif
+						debug_string(0, _("That is, the LHS is identical to the RHS."));
 						return -1;
 					}
 					found = false;
@@ -631,7 +635,9 @@ long	v;
 	token_type	*ep;
 
 #if	!SILENT
-	fprintf(gfp, _("Raising both equation sides to the power of %.*g and unfactoring...\n"), precision, 1.0 / d);
+	if (debug_level >= 0) {
+		fprintf(gfp, _("Raising both equation sides to the power of %.*g and unfactoring...\n"), precision, 1.0 / d);
+	}
 #endif
 	list_tdebug(2);
 	partial_flag = false;
@@ -643,7 +649,7 @@ long	v;
 	symb_flag = false;
 	list_tdebug(1);
 
-lonely:
+isolate:
 	ep = &tlhs[n_tlhs];
 	len2 = len1 = 0;
 	foundp = false;
@@ -741,7 +747,7 @@ end:
 	simp_loop(tlhs, &n_tlhs);
 	simp_loop(trhs, &n_trhs);
 	list_tdebug(1);
-	goto lonely;
+	goto isolate;
 }
 
 /*
@@ -771,7 +777,7 @@ long	v;	/* solve variable */
 	double		high_power = 0.0;
 
 	debug_string(1, _("Checking if equation is a polynomial equation:"));
-#if	0
+#if	DEBUG
 	if (n_tlhs != 1 || tlhs[0].kind != CONSTANT || tlhs[0].token.constant != 0.0) {
 		error_bug("poly_solve() called without a zero-solved equation!");
 	}
@@ -857,8 +863,10 @@ long	v;	/* solve variable */
 	if (high_power == 0.0)
 		return false;
 #if	!SILENT
-	list_var(v, 0);
-	fprintf(gfp, _("Equation is a degree %.*g polynomial equation in (%s).\n"), precision, high_power, var_str);
+	if (debug_level >= 0) {
+		list_var(v, 0);
+		fprintf(gfp, _("Equation is a degree %.*g polynomial equation in (%s).\n"), precision, high_power, var_str);
+	}
 #endif
 	if (a1p > trhs && (a1p - 1)->token.operatr == MINUS)
 		opx1 = MINUS;
